# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/10_gbe.sst.scoring.ipynb (unless otherwise specified).

__all__ = ['integration_methods_wrapper', 'verbruggen_quantile', 'integration_without_replacement',
           'get_integration_without_replacement_ssrts', 'mean_method_ssrt', 'get_mean_method_ssrts',
           'get_ssrt_predicted_sep_r', 'get_ssrt_sep', 'get_ssrt_predicted_joint_r', 'get_ssrt_predicted_joint']

# Cell
from .data_provider import SSTDataProvider
import pandas as pd
import numpy as np
from scipy import stats
import biuR.wrapper

# Cell
def integration_methods_wrapper(find_nth_rt):
    def wrapper(df):
        # Calculating the average SSD
        signal = df.query("is_stop==1")
        mean_ssd = signal.ssd.mean()
        # The nth RT is calculated differently by the different integration methods
        nth_rt = find_nth_rt(df) # <- This function is defined by different integration methods (see below).
        # Calculating ssrt
        ssrt = nth_rt - mean_ssd
        return ssrt
    return wrapper

#export
def verbruggen_quantile(rts, p_resp):
    R = biuR.wrapper.R()
    nth = R("""
    nth <- round(quantile(rts, probs = p_resp, type = 6))
    """, push = dict(rts = rts, p_resp = p_resp))
    return nth[0]

# Cell
@integration_methods_wrapper
def integration_without_replacement(df):
    # p_stop is calculated
    p_resp = df.query("is_stop==1").responded.mean() # p_resp matches
    # We include all valid go RTs (this means we exclude premature and late responses)
    no_signal_resp_rts = df.query("is_stop==0 and accuracy==1").rt
    nth_rt = verbruggen_quantile(no_signal_resp_rts, p_resp)
    return nth_rt

def get_integration_without_replacement_ssrts(df):
    return df.groupby('gbe_index').apply(integration_without_replacement).to_frame(name = "ssrt_integration_without_replacement")

# Cell
def mean_method_ssrt(df):
    return df.query("is_stop==0").rt.mean() - df.ssd.mean()

def get_mean_method_ssrts(df):
    return df.groupby('gbe_index').apply(mean_method_ssrt).to_frame(name = "ssrt_mean_method")

# Cell
def get_ssrt_predicted_sep_r(df):
    R = biuR.wrapper.R()
    p = R("""
    library(lmerTest)
    library(ggeffects)
    # Running the model
    control=lmerControl(optimizer = "bobyqa", optCtrl=list(maxfun=1e6))
    m = lmer(ssd_rt ~ 1 + (1 + is_stop | participant), data=df, na.action = na.exclude)
    # Extracting predicted values
    ggpredict(m, terms=c("is_stop","participant"), type="re",ci.lvl = NA)
    """,push=dict(df=df))
    m = R("""m""")
    return p, m

def get_ssrt_sep(df):
    # Creating combined variable for ssd and rt
    df['ssd_rt'] = (df['ssd']).fillna((df.query("is_stop==0").rt))
    dfs = []
    # Looping through sessions
    for session in [2,1]:
        # Extracting data for specific session and trial type
        _df = df.query('(session_number==@session)')
        # Predicting scores
        predicted, m = get_ssrt_predicted_sep_r(_df)
        predicted.columns = ['is_stop','predicted','participant']
        predicted = predicted.set_index(['participant','is_stop']).unstack()
        predicted['ssrt_predicted'] = predicted[('predicted',0.0)] - predicted[('predicted',1.0)]
        predicted = predicted[('ssrt_predicted',  '')].to_frame(name="ssrt_predicted_sep").reset_index()
        # Labeling variables
        predicted['session'] = session
        predicted['gbe_index'] = predicted.participant.astype(str) + '_%03d'%session
        predicted = predicted.set_index('gbe_index')['ssrt_predicted_sep'].to_frame()
# Combining everything into one dataframe
        dfs.append(predicted)
    perc_predicted_sep = pd.concat(dfs)
    return perc_predicted_sep, m

# Cell
def get_ssrt_predicted_joint_r(df):
    R = biuR.wrapper.R()
    p = R("""
    library(lmerTest)
    library(ggeffects)
    # Running the model
    control=lmerControl(optimizer = "bobyqa", optCtrl=list(maxfun=1e6))
    m = lmer(ssd_rt ~ 1 + (1 | is_stop / participant / session), data=df, na.action = na.exclude)
    # Extracting predicted values
    ggpredict(m, terms=c("participant", "is_stop", "session"), type="re",ci.lvl = NA)
    """,push=dict(df=df))

    m = R("""m""")
    return p, m

def get_ssrt_predicted_joint(df):
    df['ssd_rt'] = (df['ssd']).fillna((df.query("is_stop==0").rt))
    df['session'] = df.session_number.astype(str) # making session a factor
    # Predicting scores
    predicted, m = get_ssrt_predicted_joint_r(df)
    predicted.columns = ['participant','predicted','is_stop','session']
    predicted = predicted.set_index(['participant','session','is_stop']).unstack()
    predicted['ssrt_predicted_joint'] = predicted[('predicted','0')] - predicted[('predicted','1')]
    predicted = predicted[('ssrt_predicted_joint','')].to_frame(name="ssrt_predicted_joint").reset_index()
    predicted['gbe_index'] = predicted.participant.astype(str) + predicted.session.apply(lambda x: '_%03d'%int(float(x))).astype(str)
    predicted = predicted.set_index('gbe_index')['ssrt_predicted_joint'].to_frame()
    # Removing sessions that were not in initial dataframe
    predicted = predicted.loc[df.gbe_index.unique()]
    return predicted, m


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/07_gbe.sst.data_provider.ipynb (unless otherwise specified).

__all__ = ['SSTDataProvider']

# Cell
from fastcore.foundation import patch
from ..data_provider import GBEProvider
from ...data_provider import get_efficiently
import numpy as np

# Cell
class SSTDataProvider(GBEProvider):
    '''This class builds upon GBEProvider to get the working memory task data.'''
    def __init__(self, data_folder_path):
        GBEProvider.__init__(self, data_folder_path)

# Cell
@patch
def decode_sst_strings(self:SSTDataProvider, gbe_data):
    df = self.decode_gbe_strings(gbe_data, 'FruitTapGame')
        # Removing left/right distinctions
    df['rt'] = df.lefttime.astype(int) + df.righttime.astype(int)
    df['is_stop'] = (df.stop.astype(int) > 0).astype(float)
    df.loc[df.rt==0,'rt'] = np.nan # Setting 0 RTs to nan
    df['responded'] = (df.rt.isna()==False).astype(float)
    # Calculating SSD
    crw = 650 # ToDo: I'll have to double check this is correct; in Smittenaar it's reported as 500ms, but Ying used 650ms (it's correct as we use the center of response window)
    df['ssd'] = crw - df.gobaddelay.astype(int)
    df.loc[df.is_stop==False,'ssd'] = np.nan
    # Error analysis
    df['omission'] = ((df.is_stop==0) & ((df.rt.isna()) | (df.rt >= 800))).astype(float)
    df['comission'] = ((df.is_stop==1) & (df.rt.isna()==False)).astype(float)
    df['premature'] = (df.rt <= 500).astype(float)
    # Creating convenience variables and restructuring
    df['accuracy'] = df.success.astype(int)
    df = df[[
        'gbe_index',
        'trial_number',
        'anticipation',
        'is_stop','gobaddelay','ssd',
        'responded',
        'rt',
        'accuracy',
        'omission',
        'comission',
        'premature']]
    return df

# Cell
@patch
@get_efficiently
def get_sst_data(self:SSTDataProvider):
    gbe_data = self.get_gbe_data()
    df = self.decode_sst_strings(gbe_data)
    return df


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/11_gbe.rtt.scoring.ipynb (unless otherwise specified).

__all__ = ['get_percentage_gamble', 'get_perc_gamble_predicted_sep_r', 'get_perc_gamble_predicted_sep',
           'get_perc_gamble_predicted_joint_r', 'get_perc_gamble_predicted_joint', 'get_percentage_gamble',
           'get_perc_gamble_predicted_sep_r', 'get_perc_gamble_predicted_sep', 'get_perc_gamble_predicted_joint_r',
           'get_perc_gamble_predicted_joint']

# Cell
from .data_provider import RTTDataProvider
import pandas as pd
import numpy as np
from scipy import stats
import biuR.wrapper

# Cell
def get_percentage_gamble(df):
    percentage_gamble = df.groupby(['gbe_index','trial_type'])['gambled'].mean().unstack()
    percentage_gamble = percentage_gamble.add_prefix('perc_gamble_')
    return percentage_gamble

# Cell
def get_perc_gamble_predicted_sep_r(df):
    R = biuR.wrapper.R()
    p = R("""
    library(lmerTest)
    library(ggeffects)
    # Running the model
    control=glmerControl(optimizer = "bobyqa", optCtrl=list(maxfun=1e6))
    m = glmer(is_gamble ~ 1 + (1 | participant), data=df, family=binomial, control = control, na.action = na.exclude)
    # Extracting predicted values
    ggpredict(m, terms=c("participant"), type="re",ci.lvl = NA)
    """,push=dict(df=df))
    return p

def get_perc_gamble_predicted_sep(df):
    df['is_gamble'] = df.gambled.astype(int)
    dfs = []
    # Looping through trial types
    for trial_type in ['win','loss','mixed']:
        session_dfs = []
        # Looping through sessions
        for session in [2,1]:
            # Extracting data for specific session and trial type
            _df = df.query('(session_number==@session) and (trial_type==@trial_type)')
            # Predicting scores
            predicted = get_perc_gamble_predicted_sep_r(_df)
            # Labeling variables
            predicted.columns = ['participant','perc_gamble_sep_%s'%trial_type,'session']
            predicted['session'] = session
            predicted['gbe_index'] = predicted.participant.astype(str) + '_%03d'%session
            predicted = predicted.set_index('gbe_index')['perc_gamble_sep_%s'%trial_type].to_frame()
    # Combining everything into one dataframe
            session_dfs.append(predicted)
        dfs.append(pd.concat(session_dfs))
    perc_gamble_sep = pd.concat(dfs, axis = 1)
    return perc_gamble_sep

# Cell
def get_perc_gamble_predicted_joint_r(df):
    R = biuR.wrapper.R()
    p = R("""
    library(lmerTest)
    library(ggeffects)
    # Running the model
    control=glmerControl(optimizer = "bobyqa", optCtrl=list(maxfun=1e6))
    m = glmer(is_gamble ~ 1 + (1 | participant/session), data=df, family=binomial, control = control, na.action = na.exclude)
    # Extracting predicted values
    ggpredict(m, terms=c("participant","session"), type="re",ci.lvl = NA)
    """,push=dict(df=df))

    m = R("""m""")
    return p, m

def get_perc_gamble_predicted_joint(df):
    df['is_gamble'] = df.gambled.astype(int)
    dfs = []
    ms = {}
    # Looping through trial types
    for trial_type in ['win','loss','mixed']:
        # Extracting data for specific trial type
        _df = df.query('(trial_type==@trial_type)')
        _df['session'] = _df.session_number.astype(str) # making session a factor
        # Predicting scores
        predicted, m = get_perc_gamble_predicted_joint_r(_df)
        # Labeling variables
        predicted.columns = ['participant','perc_gamble_joint_%s'%trial_type,'session']
        #predicted['session'] = session
        predicted['gbe_index'] = predicted.participant.astype(str) + predicted.session.apply(lambda x: '_%03d'%int(float(x))).astype(str)
        predicted = predicted.set_index('gbe_index')['perc_gamble_joint_%s'%trial_type].to_frame()
    # Combining everything into one dataframe
        dfs.append(predicted)
        ms[trial_type] = m
    perc_predicted_sep_trial = pd.concat(dfs, axis = 1)
    # Removing sessions that were not in initial dataframe
    perc_predicted_sep_trial = perc_predicted_sep_trial.loc[df.gbe_index.unique()]
    return perc_predicted_sep_trial, ms


# Cell
from .data_provider import RTTDataProvider
import pandas as pd
import numpy as np
from scipy import stats
import biuR.wrapper

# Cell
def get_percentage_gamble(df):
    percentage_gamble = df.groupby(['gbe_index','trial_type'])['gambled'].mean().unstack()
    percentage_gamble = percentage_gamble.add_prefix('perc_gamble_')
    return percentage_gamble

# Cell
def get_perc_gamble_predicted_sep_r(df):
    R = biuR.wrapper.R()
    p = R("""
    library(lmerTest)
    library(ggeffects)
    # Running the model
    control=glmerControl(optimizer = "bobyqa", optCtrl=list(maxfun=1e6))
    m = glmer(is_gamble ~ 1 + (1 | participant), data=df, family=binomial, control = control, na.action = na.exclude)
    # Extracting predicted values
    ggpredict(m, terms=c("participant"), type="re",ci.lvl = NA)
    """,push=dict(df=df))
    return p

def get_perc_gamble_predicted_sep(df):
    df['is_gamble'] = df.gambled.astype(int)
    dfs = []
    # Looping through trial types
    for trial_type in ['win','loss','mixed']:
        session_dfs = []
        # Looping through sessions
        for session in [2,1]:
            # Extracting data for specific session and trial type
            _df = df.query('(session_number==@session) and (trial_type==@trial_type)')
            # Predicting scores
            predicted = get_perc_gamble_predicted_sep_r(_df)
            # Labeling variables
            predicted.columns = ['participant','perc_gamble_sep_%s'%trial_type,'session']
            predicted['session'] = session
            predicted['gbe_index'] = predicted.participant.astype(str) + '_%03d'%session
            predicted = predicted.set_index('gbe_index')['perc_gamble_sep_%s'%trial_type].to_frame()
    # Combining everything into one dataframe
            session_dfs.append(predicted)
        dfs.append(pd.concat(session_dfs))
    perc_gamble_sep = pd.concat(dfs, axis = 1)
    return perc_gamble_sep

# Cell
def get_perc_gamble_predicted_joint_r(df):
    R = biuR.wrapper.R()
    p = R("""
    library(lmerTest)
    library(ggeffects)
    # Running the model
    control=glmerControl(optimizer = "bobyqa", optCtrl=list(maxfun=1e6))
    m = glmer(is_gamble ~ 1 + (1 | participant/session), data=df, family=binomial, control = control, na.action = na.exclude)
    # Extracting predicted values
    ggpredict(m, terms=c("participant","session"), type="re",ci.lvl = NA)
    """,push=dict(df=df))

    m = R("""m""")
    return p, m

def get_perc_gamble_predicted_joint(df):
    df['is_gamble'] = df.gambled.astype(int)
    dfs = []
    ms = {}
    # Looping through trial types
    for trial_type in ['win','loss','mixed']:
        # Extracting data for specific trial type
        _df = df.query('(trial_type==@trial_type)')
        _df['session'] = _df.session_number.astype(str) # making session a factor
        # Predicting scores
        predicted, m = get_perc_gamble_predicted_joint_r(_df)
        # Labeling variables
        predicted.columns = ['participant','perc_gamble_joint_%s'%trial_type,'session']
        #predicted['session'] = session
        predicted['gbe_index'] = predicted.participant.astype(str) + predicted.session.apply(lambda x: '_%03d'%int(float(x))).astype(str)
        predicted = predicted.set_index('gbe_index')['perc_gamble_joint_%s'%trial_type].to_frame()
    # Combining everything into one dataframe
        dfs.append(predicted)
        ms[trial_type] = m
    perc_predicted_sep_trial = pd.concat(dfs, axis = 1)
    # Removing sessions that were not in initial dataframe
    perc_predicted_sep_trial = perc_predicted_sep_trial.loc[df.gbe_index.unique()]
    return perc_predicted_sep_trial, ms



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/19_gbe.participants.ipynb (unless otherwise specified).

__all__ = []

# Cell
%load_ext autoreload
%autoreload 2
from .data_provider import GBEProvider

import pandas as pd


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/02_gbe.data_provider.ipynb (unless otherwise specified).

__all__ = ['GBEProvider', 'determine_ema_session_numbers', 'check_repeating_sessions', 'increase_session_numbers',
           'filter_repeating_sessions']

# Cell
import numpy as np
from ..data_provider import DataProvider, get_efficiently
from fastcore.foundation import patch
import pandas as pd
from matplotlib import pyplot as plt
import re

# Cell
class GBEProvider(DataProvider):
    '''This class builds upon DataProvider and focusses on GBE data.'''
    def __init__(self, data_folder_path):
        DataProvider.__init__(self, data_folder_path)
        self.GBE_COLUMNS = ['FruitTapGame','WorkingMemoryGame','CardGame','RewardAndHappinessGame']

# Cell
@patch
def get_gbe_from_movisense(self:GBEProvider):
    """
    This function pulls GBE data from movisense and sets cancelled sessions to None.  Here, we only include rows that have at least one non-cancelled GBE game.
    """
    # Getting GBE data
    df = self.get_mov_data().query("Form == 'GreatBrainExperiment'")
    # Setting cancelled sessions to missing
    df[self.GBE_COLUMNS] = df[self.GBE_COLUMNS].replace({'{canceled": true}"':None})
    # Only included rows with at least one non-cancelled GBE game
    df = df[~df[self.GBE_COLUMNS].isna().all(axis=1)]
    df = df.sort_values(['participant','trigger_date'])
    shifted = df.groupby('participant').sampling_day.shift(1)
    df['time_since_last_gbe'] = df['sampling_day'] - shifted
    df['time_since_last_gbe'].fillna(0, inplace = True)
    df['session_number'] = df.groupby('participant').cumcount() + 1
    # Creating the trigger type variable
    replace_dict = {
    'Button Pressed Spiele starten und Initialfragen (~40min)':'initial',
    'Participant Defined Time Trigger':'ema',
    'Button Pressed Bedarfstart GBE (Ausnahme)':'optional',
    'Repeated':'optional',
    'Button Pressed Bedarfsstart GBE':'optional'}
    df['trigger_type'] = df.Trigger.apply(lambda x: x.replace('Pressed:','Pressed').split(':')[0]).replace(replace_dict)
    df['gbe_index'] = df.participant + '_' + df.session_number.apply(lambda x: '%03d'%int(x))
    df = df.set_index('gbe_index')
    return df

# Cell
@patch
def define_initial_dataset(self:GBEProvider, df):
    initial_pps = df.groupby('participant').first().sort_values(['starting_date','trigger_date']).iloc[:300].index
    df['is_initial'] = df.participant.isin(initial_pps)
    df = df.sort_index()
    return df

# Cell
@patch
def define_baseline_sessions(self:GBEProvider, df):
    first_two_sessions = df.session_number <= 2
    same_day = df.time_since_last_gbe == 0
    initial_or_optional = df.trigger_type != 'ema'
    before_ema = df.sampling_day < 22 # Ema starts on sampling day 22, so we only consider baseline sessions that happened before that
    df['is_baseline'] = first_two_sessions & same_day & initial_or_optional & before_ema
    return df

# Cell
@patch
def find_gbe_game_starting_time(self:GBEProvider, s):
    '''As baseline GBEs have the same trigger date, we first have to parse the time from the GBE data.'''
    nan_time = pd.to_datetime(np.nan)
    try:
        gbe_columns = s.split(',')

    except:
        return nan_time
    if 'timestarted' not in gbe_columns:
        return nan_time
    time_started_index = gbe_columns.index('timestarted')
    time_started = pd.to_datetime(s.split('\n')[1].split(',')[time_started_index][2:-2])
    return time_started



@patch
def check_time_between_sessions(self:GBEProvider, df):
    time_1 = pd.to_datetime(np.nan)
    time_2 = pd.to_datetime(np.nan)
    if len(df)==2:
        time_1 = self.find_gbe_game_starting_time(df.iloc[0])
        time_2 = self.find_gbe_game_starting_time(df.iloc[1])
    try:
        return (time_2 - time_1).seconds/60
    except:
        return np.nan

# Cell
def determine_ema_session_numbers(_df):
    '''Ema sessions start the earliest after sampling day 22.  The following year is split into 28-day intervals, each corresponding to one EMA session.'''
    _df['ema_session'] = pd.cut(_df.sampling_day, np.arange(21, 392, 28), labels = [x for x in range(1,14)]).astype(float)
    return _df

def check_repeating_sessions(_df):
    '''Sometimes EMA sessions end up happening in the same 28 days interval.  In these cases, participants therefore have repeating sessions.  If a repeating session is one the same day, it should certainly be removed.  However, if a repeating session happens far enough from other sessions and the subsequent session is missing, we can assume that this session is actually a real session.  Although, the nominal distance between sessions is 28 days, participants can postpone sessions, thus reducing the time to the next session.  Specifying at least 14 one days between sessions, therefore seems reasonable.'''
    _df['ema_session_repeating'] = ((_df.ema_session.astype(float) - _df.ema_session.astype(float).shift(1))==0)
    _df['total_ema_sessions_completed'] = _df.groupby('participant').ema_session.transform(lambda x:len(x.dropna().unique()))
    _df['time_since_last_session'] = ((_df.sampling_day - _df.groupby('participant').sampling_day.shift(1)))
    _df['time_to_next_session'] = ((_df.sampling_day - _df.groupby('participant').sampling_day.shift(-1))*-1)
    # Deciding wether to increase session number
    sessions_missing = _df['total_ema_sessions_completed'] < 13
    window = 14
    enough_distance = (_df.time_since_last_session>window) & (_df.time_to_next_session>window)
    _df['increase_session_number'] = (_df.ema_session_repeating & enough_distance & sessions_missing).astype(int)
    return _df

def increase_session_numbers(_df):
    _df['ema_session'] = _df.ema_session + _df.increase_session_number
    return(_df)

def filter_repeating_sessions(_df):
    _df = _df.query('ema_session_repeating==False')
    _df = _df.query('is_baseline or (ema_session.isna() == False)')
    return _df

@patch
def fix_repeating_ema_sessions(self:GBEProvider, _df):
    _df = determine_ema_session_numbers(_df)
    old_df = _df
    for i in range(13):
        _df = check_repeating_sessions(_df)
        _df = increase_session_numbers(_df)
    old_sessions = _df.index

    _df = filter_repeating_sessions(_df)
    new_sessions = _df.index
    removed_sessions = list((set(old_sessions)-set(new_sessions)))
    #return old_df.loc[removed_sessions]
    removed_participants = list(set([s.split('_')[0] for s in removed_sessions]))
    _df = filter_repeating_sessions(_df)
    _df = check_repeating_sessions(_df)
    print("%d participants had unplanned sessions.  %d sessions (%.2f%%) were removed."%(len(removed_participants), len(removed_sessions), len(old_df)/len(removed_sessions)))
    _df['session_number_ex_protocol'] = _df.session_number
    _df['session_number'] = (_df.ema_session+2).fillna(_df.session_number)
    return _df

# Cell
@patch
@get_efficiently
def get_gbe_data(self:GBEProvider):
    df = self.get_gbe_from_movisense()
    df = self.define_initial_dataset(df)
    print(df.is_initial.isna().mean())
    df = self.define_baseline_sessions(df)
    print(df.is_initial.isna().mean())
    df = self.fix_repeating_ema_sessions(df)
    print(df.is_initial.isna().mean())
    df = df.reset_index()
    df['gbe_index'] = df.participant + '_' + df.session_number.apply(lambda x: '%03d'%int(x))
    df = df.set_index('gbe_index')
    return df

# Cell
@patch
def get_initial_baseline(self:GBEProvider):
    df = self.get_gbe_data()
    return df.query("is_initial and is_baseline")

@patch
def get_replication_baseline(self:GBEProvider):
    df = self.get_gbe_data()
    return df.query("(is_baseline and (not initial)")

# Cell
@patch
def decode_gbe_string(self:GBEProvider, s):
    """This function function turns one gbe output string into a dataframe"""
    def replace(g):
        return g.group(0).replace(',', '|')
    s = re.sub(r'\[.*?\]', replace, s) # The comma separated string can contain comma separated list items
    columns, df = s.replace('","',';').replace('"','').split('\n')
    df = pd.DataFrame([column.split(',') for column in df.split(';')][:-1]).transpose().ffill().iloc[:-1]
    df.columns = [c.replace('tr_','') for c in columns.split(',')[:-1]]
    def to_datetime(x):
        return pd.to_datetime(x[1:-1])
    if 'timestarted' in df.columns:
        df['timestarted'] = df.timestarted.apply(to_datetime)
        df['timesubmitted'] = df.timesubmitted.apply(to_datetime)
        df.drop(columns = 'appversion', inplace = True)
    else:
        df['timestarted'] = pd.to_datetime(np.nan)
        df['timesubmitted'] = pd.to_datetime(np.nan)
    return df

# Cell
@patch
def decode_gbe_strings(self:GBEProvider, df, column):
    '''This function turns all gbe output strings in a column into dataframes and concatenates them.'''
    df = df[~df[column].isna()] # Selecting non nan data
    #gbe_data = pd.concat(df.set_index(['participant','session_number'])[column].apply(self.decode_gbe_string).values, keys = df.index)
    #gbe_data = pd.concat(df[column].apply(self.decode_gbe_string).values)
    df = pd.concat(df[column].apply(self.decode_gbe_string).values, keys = df.index)
    df.index.rename('trial_number',level = 1, inplace = True)
    df = df.reset_index()
    df['trial_number'] = df.trial_number + 1
    return df


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/08_gbe.rtt.data_provider.ipynb (unless otherwise specified).

__all__ = ['RTTDataProvider']

# Cell
from fastcore.foundation import patch
from ..data_provider import GBEProvider
from ...data_provider import get_efficiently
import numpy as np

# Cell
class RTTDataProvider(GBEProvider):
    '''This class builds upon GBEProvider to get the working memory task data.'''
    def __init__(self, data_folder_path):
        GBEProvider.__init__(self, data_folder_path)

# Cell
@patch
def decode_rtt_strings(self:RTTDataProvider, gbe_data):
    df = self.decode_gbe_strings(gbe_data, "RewardAndHappinessGame")
    # Calculating outcome variables
    df['trial_type'] = np.nan
    df.loc[(df.choiceamount.astype(float) < 0),'trial_type'] = 'loss'
    df.loc[(df.choiceamount.astype(float) > 0),'trial_type'] = 'win'
    df.loc[(df.choiceamount.astype(float) == 0),'trial_type'] = 'mixed'
    df['result'] = df.trialresult.replace({'1':'gain', '2': 'loss', '3':'certain'})
    df['gambled'] = df.result!='certain'
    # Filtering variables of interest
    df = df[["gbe_index","trial_number","timestarted", "timesubmitted", "choiceamount", "decisiontime", "happiness", "happinessstart", "happinesstime" ,"spinnerangle", "spinnerloseamount", "spinnerwinamount", "spintime", "trial_type","trialresult","gambled"]]
    return df

# Cell
@patch
@get_efficiently
def get_rtt_data(self:RTTDataProvider):
    gbe_data = self.get_gbe_data()
    df = self.decode_rtt_strings(gbe_data)
    return df


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/20_plotting.ipynb (unless otherwise specified).

__all__ = ['custom_params', 'icc_comparison_plot']

# Cell
import seaborn as sns
import pandas as pd
from matplotlib import pyplot as plt
custom_params = {"axes.spines.right": False, "axes.spines.top": False}
sns.set_theme(style="white", font_scale = 1.3, rc=custom_params)

# Cell
def icc_comparison_plot(iccs,
                        icc_type = "ICC1",
                       method_labels = {'mean':'Based on mean method',
                      'trial_sep':'Based on predicted values (separate models)',
                      'trial_joint':'Based on predicted values (joint model)',
                      'model_calc':'Based on model-calculated variances'
                      },
                        qualitative_labels = False,
                        x_offset = 3.5,
                        font_scale = 1.3,
                        **catplot_kwgs):
    custom_params = {"axes.spines.right": False, "axes.spines.top": False}
    sns.set_theme(style="white", font_scale = font_scale, rc=custom_params)
    icc_type = "ICC1"
    plot_df = pd.DataFrame({})
    for k, v in iccs.items():
        if 'iccs' in v.keys():
            icc = v['iccs'][0].query('type=="%s"'%icc_type)['ICC'].iloc[0]
        else:
            icc = v['icc']
        plot_df.loc[k,'icc'] = icc

    if 'aspect' not in catplot_kwgs.keys():
        catplot_kwgs['aspect'] = 1.3
    if 'height' not in catplot_kwgs.keys():
        catplot_kwgs['height'] = 6

    plot_df = plot_df.reset_index()
    plot_df.rename(columns = {'index':'category','icc':"Test-retest reliability (%s)"%icc_type}, inplace = True)
    plot_df['trial_type'] = plot_df.category.apply(lambda x: x.split('_')[-1])
    plot_df['Method'] = plot_df.category.apply(lambda x: '_'.join(x.split('_')[:-1]))
    plot_df.Method.replace(method_labels, inplace = True)
    g = sns.catplot(x = "trial_type",       # x variable name
                y = "Test-retest reliability (%s)"%icc_type,       # y variable name
                hue = "Method",  # group variable name
                data = plot_df,     # dataframe to plot
                kind = "bar",
                    legend = False,
               ci = None,
                **catplot_kwgs)
    g.set(ylim=(0, 1))
    ax = g.axes[0][0]

    if qualitative_labels:
        y_offset = -.07
        x_offset = x_offset
        alignment = 'left'
        plt.axhline(.5, color = 'grey', linestyle = '--',xmax = 1)
        plt.axhline(.75, color = 'grey', linestyle = '--',xmax = 1)
        plt.annotate("poor reliability", (0+x_offset,.5+y_offset),horizontalalignment = alignment)
        plt.annotate("moderate reliability", (0+ x_offset,.67+y_offset),horizontalalignment = alignment)
        plt.annotate("good reliability", (0+x_offset,.86+y_offset),horizontalalignment = alignment)
    plt.legend(loc='upper left', bbox_to_anchor=(1, 1.1))

    return g


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/26_gbe.baseline_drinking_separate.ipynb (unless otherwise specified).

__all__ = []

# Cell
%load_ext autoreload
%autoreload 2
from .ist.data_provider import ISTDataProvider
from .wm.data_provider import WMDataProvider
from .sst.data_provider import SSTDataProvider
from .rtt.data_provider import RTTDataProvider

import trr265.gbe.ist.scoring as ist_scoring
import trr265.gbe.wm.scoring as wm_scoring
import trr265.gbe.sst.scoring as sst_scoring
import trr265.gbe.rtt.scoring as rtt_scoring

import pandas as pd

# Cell
%load_ext autoreload
%autoreload 2
from .ist.data_provider import ISTDataProvider
from .wm.data_provider import WMDataProvider
from .sst.data_provider import SSTDataProvider
from .rtt.data_provider import RTTDataProvider

import trr265.gbe.ist.scoring as ist_scoring
import trr265.gbe.wm.scoring as wm_scoring
import trr265.gbe.sst.scoring as sst_scoring
import trr265.gbe.rtt.scoring as rtt_scoring

import pandas as pd


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/21_validation_tasks.ipynb (unless otherwise specified).

__all__ = []

# Cell
%load_ext autoreload
%autoreload 2
from .data_provider import DataProvider
import json
import pandas as pd
import seaborn as sns


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/22_gbe.rtt.singular_fit_inspection.ipynb (unless otherwise specified).

__all__ = []

# Cell
%load_ext autoreload
%autoreload 2
from .data_provider import RTTDataProvider
import trr265.gbe.rtt.scoring as rtt_scoring
import trr265.stats as stats
import pandas as pd




# Cell
%load_ext autoreload
%autoreload 2
from .gbe.ist.data_provider import ISTDataProvider
from .gbe.wm.data_provider import WMDataProvider
from .gbe.sst.data_provider import SSTDataProvider
from .gbe.rtt.data_provider import RTTDataProvider

import trr265.gbe.ist.scoring as ist_scoring
import trr265.gbe.wm.scoring as wm_scoring
import trr265.gbe.sst.scoring as sst_scoring
import trr265.gbe.rtt.scoring as rtt_scoring

import pandas as pd


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/01_data_provider.ipynb (unless otherwise specified).

__all__ = ['DataProvider', 'get_efficiently']

# Cell
from bs4 import BeautifulSoup as bs
import numpy as np
import os
import pandas as pd
from fastcore.foundation import patch

# Cell
class DataProvider():
    def __init__(self, data_folder_path):
        self.data_folder_path = data_folder_path
        self.raw = os.path.join(data_folder_path, 'raw')
        self.external = os.path.join(data_folder_path, 'external')
        self.interim = os.path.join(data_folder_path, 'interim')
        self.processed = os.path.join(data_folder_path, 'processed')
        # Checking if folder paths exist
        assert os.path.isdir(self.external), "External data folder not found."
        assert os.path.isdir(self.raw), "Raw data folder not found."
        assert os.path.isdir(self.interim), "Interim data folder not found."
        assert os.path.isdir(self.processed), "Processed data folder not found."
        # Phone screening files
        self.phonescreening_data_path = os.path.join(self.raw, "phonescreening.csv")
        self.phone_codebook_path = os.path.join(self.external, "phone_codebook.html")
        # Basic assessment files
        self.ba_codebook_path = os.path.join(self.external, "ba_codebook.html")
        self.ba_data_path = os.path.join(self.raw, "ba.csv")
        self.b07_participants_path = os.path.join(self.external, "b7_participants.xlsx")
        # Movisense data
        self.mov_berlin_path = os.path.join(self.raw, "mov_data_b.csv")
        self.mov_dresden_path = os.path.join(self.raw, "mov_data_d.csv")
        self.mov_mannheim_path = os.path.join(self.raw, "mov_data_m.csv")
        self.mov_berlin_starting_dates_path = os.path.join(self.raw, "starting_dates_b.html")
        self.mov_dresden_starting_dates_path = os.path.join(self.raw, "starting_dates_d.html")
        self.mov_mannheim_starting_dates_path = os.path.join(self.raw, "starting_dates_m.html")
        self.alcohol_per_drink_path = os.path.join(self.external,'alcohol_per_drink.csv')


#export
def get_efficiently(func):
    """
    This decorator wraps around functions that get data and handles data storage.
    If the output from the function hasn't been stored yet, it stores it in "[path_to_interim]/[function_name_without_get].parquet"
    If the output from the function has been stored already, it loads the stored file instead of running the function (unless update is specified as True)
    """
    def w(*args, update = False, columns = None, path = None, **kw):
        _self = args[0] # Getting self to grab interim path from DataProvider
        var_name = func.__name__.replace('__get_','').replace('get_','')
        file_path = os.path.join(_self.interim, "%s.parquet"%var_name)
        if os.path.exists(file_path) and (update == False):
            result =  pd.read_parquet(file_path, columns = columns)
        else:
            print("Preparing %s"%var_name)
            result = func(_self)
            result.to_parquet(file_path)
        return result
    w.__wrapped__ = func # Specifying the wrapped function for inspection
    w.__doc__ = func.__doc__
    w.__name__ = func.__name__
    w.__annotations__ = {'cls':DataProvider, 'as_prop':False} # Adding parameters to make this work with @patch
    return w

# Cell
@patch
def store_interim(self:DataProvider, df, filename):
    path = os.path.join(self.interim,"%s.parquet"%filename)
    df.to_parquet(path)

# Cell
@patch
def load_interim(self:DataProvider, filename):
    return pd.read_parquet(os.path.join(self.interim,"%s.parquet"%filename))

# Cell
@patch
@get_efficiently
def get_phone_codebook(self:DataProvider):
    tables = pd.read_html(open(self.phone_codebook_path,'r').read())
    df = tables[1]
    # Note that str.contains fills NaN values with nan, which can lead to strange results during filtering
    df = df[df.LabelHinweistext.str.contains('Fragebogen:',na=False)==False]
    df = df.set_index('#')
    # Parsing variable name
    df['variable'] = df["Variable / Feldname"].apply(lambda x: x.split(' ')[0])
    # Parsing condition under which variable is displayed
    df['condition'] = df["Variable / Feldname"].apply(lambda x: ' '.join(x.split(' ')[1:]).strip() if len(x.split(' '))>1 else '')
    df['condition'] = df.condition.apply(lambda x: x.replace('Zeige das Feld nur wenn:  ',''))
    # Parsing labels for numerical data
    df['labels'] = np.nan
    labels = tables[2:-1]
    try:
        labels = [dict(zip(l[0],l[1])) for l in labels]
    except:
        display(table)
    searchfor = ["radio","dropdown","yesno","checkbox"]
    with_table = df['Feld Attribute (Feld-Typ, Prüfung, Auswahlen, Verzweigungslogik, Berechnungen, usw.)'].str.contains('|'.join(searchfor))
    df.loc[with_table,'labels'] = labels
    df = df.astype(str)
    return df

# Cell
@patch
def determine_phone_b07(self:DataProvider, df):
    # Some initial fixes
    df.loc[df.center=='d','screen_caller'] = df.loc[df.center=='d','screen_caller'].str.lower().str.strip().replace('leo','leonard visser').replace('sebastian mörcke','sebastian möricke').replace('jessica zimmerman','jessica zimmermann').replace('miriam-sophie petasch','miriam petasch').replace('dorothee','dorothee scheuermann')
    # Cleaning screener list
    dd_screeners = df[(df.center=='d')&(df.screen_caller.isna()==False)].screen_caller.unique()
    def clean_screeners(dd_screeners):
        dd_screeners = [y  for x in dd_screeners for y in x.split('+')]
        dd_screeners = [y  for x in dd_screeners for y in x.split(',')]
        dd_screeners = [y  for x in dd_screeners for y in x.split('und')]
        dd_screeners = [y.replace('(15.02.21)','')  for x in dd_screeners for y in x.split('/')]
        dd_screeners = [y.replace(')','').strip().lower()  for x in dd_screeners for y in x.split('(')]
        dd_screeners = sorted(list(set(dd_screeners)))
        return dd_screeners
    dd_screeners = clean_screeners(dd_screeners)

    b07_screeners = ['ann-kathrin stock','charlotte blum','josephine kirschgens','klara macht','borchardt','marta ledro','miriam petasch','mona hofmann','theo tester']
    s01_screeners = ['esther preuschhof', 'miriam schmitz', 'sebastian möricke', 'jessica zimmermann', 'leonard visser', 'anna-lena lünert', 'anne dörfler', 'dominic reichert', 'maike borchardt', 'dorothee scheuermann', 'paula böhlmann', 'alice','josephine kirschgens maxi stiller', 'maria schießl', '22.10.2021', 'sascha', '03.08.2021', 'mariana plumbohm', 'caroline neumer', '04.08.2021', 'maxi stiller', 'sacsha', '09.08.2021', 'ml', 'charlotte', 'charlotte heinze', 'shereen', 'test', "charlotte heinze", 'benedikt']
    known_dd_screeners = list(b07_screeners+s01_screeners)
    dd_screeners = df[(df.center=='d')&(df.screen_caller.isna()==False)].screen_caller.unique()
    # Checking if all Dresden phone screeners are accounted for
    assert df[(df.center=='d')&(df.screen_caller)].screen_caller.str.contains('|'.join(known_dd_screeners)).mean()==1, "Unknown Dresden phone screener: %s"%', '.join(set(clean_screeners(dd_screeners))-set(known_dd_screeners))
    # In general, if a screener from a project was involved, it was screened for that project
    df['screened_for_b07'] = (df.center=='d') & (df.screen_caller.str.contains('|'.join(b07_screeners)))
    df['screened_for_s01'] = (df.center!='d') | (df.screen_caller.str.contains('|'.join(s01_screeners)))

    # We also exclude participants screened for C02 in Berlin
    df.loc[(df.screen_purpose == 4) & (df.center=='b'), 'screened_for_s01'] = False

    # Additionally, we also set it to true if it was specifically set
    df.loc[df.screen_site_dd == 1, 'screened_for_s01'] = True
    df.loc[df.screen_site_dd == 3, 'screened_for_s01'] = True
    df.loc[df.screen_site_dd == 2, 'screened_for_b07'] = True
    df.loc[df.screen_site_dd == 3, 'screened_for_b07'] = True
    return df

# Cell
@patch
def check_participant_id(self:DataProvider,x):
    '''This function checks whether a participant ID is numerical and lower than 20000.'''
    if str(x) == x:
        if x.isnumeric():
            x = float(x)
        else:
            return False
    if x > 20000:
        return False
    return True

# Cell
@patch
def test_check_participant_id(self:DataProvider):
    failed = dp.check_participant_id('test10') == False # Example of a bad participant ID
    passed = dp.check_participant_id('100') == True # Example of a good participant ID
    return failed and passed

# Cell
@patch
def set_dtypes(self:DataProvider, data, codebook):
    def number_or_nan(x):
        try:
            float(x)
            return x
        except:
            return np.nan
    '''This function automatically adjust data types of redcap data based on the redcap codebooks'''
    # Parsing type
    codebook['type'] = codebook["Feld Attribute (Feld-Typ, Prüfung, Auswahlen, Verzweigungslogik, Berechnungen, usw.)"].apply(lambda x: x.split(',')[0])
    # Descriptives (not in data)
    desc_columns = list(codebook[codebook.type.str.contains('descriptive')].variable)
    # Datetime
    dt_columns = codebook[(codebook.type.isin(['text (datetime_dmy)','text (date_dmy)']))].variable
    dt_columns = list(set(data.columns).intersection(dt_columns))
    # Numerical
    num_columns = []
    num_columns += list(codebook[codebook.type.str.contains('calc')].variable)
    num_columns += list(codebook[codebook.type.str.contains('checkbox')].variable)
    num_columns += list(codebook[codebook.type.str.contains('radio')].variable)
    num_columns += list(codebook[codebook.type.str.contains('text \(number')].variable)
    num_columns += list(codebook[codebook.type.str.contains('yesno')].variable)
    num_columns += list(codebook[codebook.type.str.contains('dropdown')].variable)
    num_columns += list(codebook[codebook.type.str.contains('slider')].variable)
    num_columns = list(set(data.columns).intersection(num_columns))
    # Text
    text_columns = []
    text_columns += list(codebook[(codebook.type.str.contains('text')) & (~codebook.type.str.contains('date_dmy|datetime_dmy'))].variable)
    text_columns += list(codebook[(codebook.type.str.contains('notes'))].variable)
    text_columns += list(codebook[(codebook.type.str.contains('file'))].variable)
    text_columns = list(set(data.columns).intersection(text_columns))
    assert len(set(num_columns).intersection(set(dt_columns)))==0, set(num_columns).intersection(set(dt_columns))
    assert len(set(text_columns).intersection(set(dt_columns)))==0, set(text_columns).intersection(set(dt_columns))



    for c in num_columns:
        data[c].replace("A 'MySQL server has gone away' error was detected.  It is possible that there was an actual database issue, but it is more likely that REDCap detected this request as a duplicate and killed it.", np.nan, inplace = True)
        try:
            data[c] = data[c].astype(float)
        except:
            data[c] = data[c].apply(number_or_nan).astype(float)
            print("Values with wrong dtype in %s"%c)
    data[text_columns] = data[text_columns].astype(str).replace('nan',np.nan)

    for c in dt_columns:
        data[c] = pd.to_datetime(data[c])
    return data

# Cell
@patch
@get_efficiently
def get_phone_data(self:DataProvider):
    df = pd.read_csv(self.phonescreening_data_path,
                     na_values = ["A 'MySQL server has gone away' error was detected.  It is possible that there was an actual database issue, but it is more likely that REDCap detected this request as a duplicate and killed it."]
                    )
    remove = ['050571', '307493', '345678', '715736', 'Ihloff', 'test',
       'test002', 'test003', 'test004', 'test005', 'test01', 'test02',
       'test03', 'test0722', 'test1', 'test34', 'test999', 'test2020',
       'test20201', 'test345345', 'testt', 'test_10', 'test_11_26',
       'test_neu', 'xx956','050262', '050335', '050402', '050416', '051005', '294932', '891752080', '898922719', '898922899', '917702419', '01627712983', 'meow', 'test0022', 'test246', 'test5647', 'test22222', 'test41514', 'testtt', 'test_057', 'tets','898923271', 'test001', 'test006', 'test007', 'test008', 'test11', 'test_23_12', 'test_n','50744', 'test0001a', 'test004', 'test03', 'tets', 'test_neu', 'xx956','050262', '050335', '050402', '050416', '051005', '294932', '891752080', '898922719', '898922899', '917702419', '01627712983', 'meow', 'test0022', 'test246', 'test5647', 'test22222', 'test41514', 'testtt', 'test_057']
    df = df[~df.participant_id.astype(str).isin(remove)]

    bad_ids = df[~df.participant_id.apply(self.check_participant_id)].participant_id.unique()
    assert len(bad_ids)==0, "Bad participant IDs (should be added to remove): %s"%', '.join(["'%s'"%b for b in bad_ids])
    self.get_phone_codebook()
    df = self.set_dtypes(df, self.get_phone_codebook())
    df['participant_id'] = df.participant_id.astype(int)
    df['center'] = df.screen_site.replace({1:'b',2:'d',3:'m'})
    df['screen_date'] = pd.to_datetime(df['screen_date'], errors = 'coerce')
    #display(df[df.screen_caller.isna()])
    df = self.determine_phone_b07(df)
    return df

# Cell
@patch
@get_efficiently
def get_ba_codebook(self:DataProvider):
    tables = pd.read_html(open(self.ba_codebook_path,"r", encoding ='UTF-8').read())
    df = tables[1]
    # Note that str.contains fills NaN values with nan, which can lead to strange results during filtering
    df = df[df.LabelHinweistext.str.contains('Fragebogen:',na=False)==False]
    df = df.set_index('#')
    # Parsing variable name
    df['variable'] = df["Variable / Feldname"].apply(lambda x: x.split(' ')[0])
    # Parsing condition under which variable is displayed
    df['condition'] = df["Variable / Feldname"].apply(lambda x: ' '.join(x.split(' ')[1:]).strip() if len(x.split(' '))>1 else '')
    df['condition'] = df.condition.apply(lambda x: x.replace('Zeige das Feld nur wenn:  ',''))
    # Parsing labels for numerical data
    df['labels'] = np.nan
    labels = tables[2:-1]
    try:
        labels = [dict(zip(l[0],l[1])) for l in labels]
    except:
        display(table)
    searchfor = ["radio","dropdown","yesno","checkbox"]
    with_table = df['Feld Attribute (Feld-Typ, Prüfung, Auswahlen, Verzweigungslogik, Berechnungen, usw.)'].str.contains('|'.join(searchfor))
    df.loc[with_table,'labels'] = labels
    df = df.astype(str)
    return df

# Cell
@patch
@get_efficiently
def get_ba_data(self:DataProvider):
    '''This function reads in baseline data from redcap, filters out pilot data, and creates movisens IDs.'''
    df = pd.read_csv(self.ba_data_path)
    df['center'] = df.groupby('participant_id').bx_center.transform(lambda x: x.ffill().bfill())
    df['center'] = df.center.replace({1:'b',2:'d',3:'m'})
    # Creating new movisense IDs (adding center prefix to movisense IDs)
    for old_id in ['bx_movisens','bx_movisens_old','bx_movisens_old_2']:
        new_id = old_id.replace('bx_','').replace('movisens','mov_id')
        df[new_id] = df.groupby('participant_id')[old_id].transform(lambda x: x.ffill().bfill())
        df[new_id] = df.center + df[new_id].astype('str').str.strip('0').str.strip('.').apply(lambda x: x.zfill(3))
        df[new_id].fillna('nan',inplace = True)
        df.loc[df[new_id].str.contains('nan'),new_id] = np.nan
    # Removing test participants
    remove = ['050744', 'hdfghadgfh', 'LindaEngel', 'test', 'Test001', 'Test001a', 'test0011', 'test0012', 'test0013', 'test0014', 'test0015', 'test002', 'test00229', 'test007', 'test01', 'test012', 'test013', 'test1', 'test2', 'test4', 'test12', 'test999', 'test2021', 'test345345', 'testneu', 'testtest', 'test_0720', 'test_10', 'test_GA', 'Test_JH','test0016','891752080', 'pipingTest', 'test0001', 'test00012', 'test0012a', 'test0015a', 'test0017', 'test10', 'test20212', 'testJohn01', 'test_00213', 'test_00233', 'test_00271', 'test_003', 'test_004', 'test_11_26', 'Test_MS','898922899', 'tesst', 'test0002', 'test0908', 'test092384750398475', 'test43', 'test123', 'test1233', 'test3425', 'test123456', 'test1234567', 'testfu3', 'test_888', 'test_999', 'test_98375983745', 'Test_Übung','050335', 'test003', 'test02', 'test111', 'test1111', 'test1234','test0000', 'test_CH','50744', 'test0001a', 'test004', 'test03', 'tets','test009', 'test22', 'test912', 'test3003', 'test11111', 'test_MH']
    df = df[~df.participant_id.astype(str).isin(remove)]
    # Checking participant ids (to find new test participants)
    bad_ids = df[~df.participant_id.apply(self.check_participant_id)].participant_id.unique()
    assert len(bad_ids)==0, "Bad participant IDs (should be added to remove): %s"%', '.join(["'%s'"%b for b in bad_ids])
    # labeling B07 participant
    b07_pps = pd.read_excel(self.b07_participants_path)['Participant ID'].astype(str)
    df['is_b07'] = False
    df.loc[df.participant_id.isin(b07_pps),'is_b07'] = True
    # Setting dtypes based on codebook
    df = self.set_dtypes(df, self.get_ba_codebook())
    # Creating convenience variables
    df['is_female'] = df.screen_gender.replace({1:1,2:0,3:np.nan})
    # Filling in missings from baseline
    df['is_female'].fillna(df.bx_sozio_gender.replace({1:0,2:1,3:np.nan}), inplace = True)
    df['is_female'] = df.groupby('participant_id')['is_female'].transform(lambda x: x.ffill().bfill())
    df['is_female'] = df['is_female'].astype(float)

    return df

# Cell
@patch
def get_baseline_drinking_data(self:DataProvider):
    # Getting relevant data
    ba = self.get_ba_data(columns = ['participant_id','redcap_event_name','mov_id','bx_qf_alc_01','bx_qf_alc_02','bx_qf1_sum']).query("redcap_event_name=='erhebungszeitpunkt_arm_1'")
    # Correct one variable for one participant.  This participant reported drinking per three months but the data as logged as drinking per week
    ba.loc[(ba.participant_id=='11303') & (ba.bx_qf_alc_02==2),'bx_qf_alc_02'] = 1
    ba['drinking_days_last_three_month'] = ba['bx_qf_alc_01'].astype(float) * ba['bx_qf_alc_02'].replace({2:12})
    ba['drinks_per_drinking_day_last_three_month'] = ba['bx_qf1_sum']
    ba['drinks_per_day_last_three_month'] = (ba['drinking_days_last_three_month'] * ba['bx_qf1_sum'])/90
    standard_last_three = ba[~ba.drinks_per_day_last_three_month.isnull()][['mov_id','drinks_per_day_last_three_month','drinks_per_drinking_day_last_three_month','drinking_days_last_three_month']]
    standard_last_three.columns = ['participant','last_three_month','drinks_per_drinking_day_last_three_month','drinking_days_last_three_month']
    standard_last_three = standard_last_three.groupby('participant').first()
    return standard_last_three

# Cell
@patch
def get_duplicate_mov_ids(self:DataProvider):
    '''This function creates a dictionary mapping old to new movisens IDs.'''
    df = self.get_ba_data()
    replace_dict_1 = dict(zip(df.mov_id_old, df.mov_id))
    replace_dict_2 = dict(zip(df.mov_id_old_2, df.mov_id))
    replace_dict = {**replace_dict_1, **replace_dict_2}
    try:
        del replace_dict[np.nan]
    except:
        pass
    if None in replace_dict.keys():
        del replace_dict[None]
    replace_dict['d033'] = 'd092' # This participant's data is currently missing in redcap, but they did change ID from 33 to 92
    return replace_dict

# Cell
@patch
@get_efficiently
def get_mov_data(self:DataProvider):
    """
    This function gets Movisense data
    1) We create unique participnat IDs (e.g. "b001"; this is necessary as sites use overapping IDs)
    2) We merge double IDs, so participants with two IDs only have one (for this duplicate_ids.csv has to be updated)
    3) We remove pilot participants
    4) We get starting dates (from the participant overviews in movisense; downloaded as html)
    5) We calculate sampling days and end dates based on the starting dates
    """
    # Loading raw data
    mov_berlin = pd.read_csv(self.mov_berlin_path, sep = ';')
    mov_dresden = pd.read_csv(self.mov_dresden_path, sep = ';')
    mov_mannheim = pd.read_csv(self.mov_mannheim_path, sep = ';')

    # Merging (participant numbers repeat so we add the first letter of location)
    mov_berlin['location'] = 'berlin'
    mov_dresden['location'] = 'dresden'
    mov_mannheim['location'] = 'mannheim'
    df = pd.concat([mov_berlin,mov_dresden,mov_mannheim])
    df['participant'] =  df['location'].str[0] + df.Participant.apply(lambda x: '%03d'%int(x))
    df.drop(columns = 'Participant', inplace = True) # Dropping old participant column to avoid mistakes
    df['trigger_date'] = pd.to_datetime(df.Trigger_date + ' ' + df.Trigger_time)

    # Merging double IDs (for participants with several movisense IDs)
    df['participant'] = df.participant.replace(self.get_duplicate_mov_ids())

    # Removing pilot participants
    df = df[~df.participant.astype(str).str.contains('test')]
    df = df[~df.participant.isin(['m157', 'b010', 'b006', 'd001', 'd002', 'd042', 'm024', 'm028', 'm071', 'm079', 'm107'])]


    # Adding starting dates to get sampling days
    def get_starting_dates(path, pp_prefix = ''):
        soup = bs(open(path).read())
        ids = [int(x.text) for x in soup.find_all("td", class_ = 'simpleId')]
        c_dates = [x.find_all("span")[0]['title'] for x in soup.find_all("td", class_ = 'coupleDate')]
        s_dates = [x['value'] for x in soup.find_all("input", class_ = 'dp startDate')]
        df = pd.DataFrame({'participant':ids,'coupling_date':c_dates,'starting_date':s_dates})
        df['coupling_date'] = pd.to_datetime(df.coupling_date)
        df['starting_date'] = pd.to_datetime(df.starting_date)
        df.starting_date.fillna(df.coupling_date,inplace = True)
        df['participant'] = pp_prefix + df.participant.apply(lambda x: '%03d'%int(x))
        return df

    starting_dates = pd.concat([
    get_starting_dates(self.mov_berlin_starting_dates_path, 'b'),
    get_starting_dates(self.mov_dresden_starting_dates_path, 'd'),
    get_starting_dates(self.mov_mannheim_starting_dates_path, 'm')
    ])
    # For participants with several movisense IDs we use the first coupling date
    starting_dates.participant.replace(self.get_duplicate_mov_ids(), inplace = True)
    starting_dates = starting_dates.groupby('participant')[['starting_date','coupling_date']].min().reset_index()
    df = df.merge(starting_dates, on="participant", how = 'left', indicator = True)
    # Checking if starting dates were downloaded
    if "left_only" in df._merge.unique():
        no_starting_dates = df.query('_merge == "left_only"').participant.unique()
        print("Starting dates missing for participants below.  Did you download the participant overviews as html?", no_starting_dates)
    # Calculating movisense sampling day, adding date and end_date
    df['sampling_day'] = (df['trigger_date'] - df['starting_date']).dt.days + 1
    df['date'] = df.trigger_date.dt.date
    df['end_date'] = df.date + pd.DateOffset(days = 365)
    df.index.rename('mov_index',inplace = True)
    # Adding redcap IDs
    ids_table = self.get_ba_data()[['participant_id','mov_id']].query('mov_id==mov_id').groupby('mov_id').first()
    ids_table.columns = ['redcap_id']
    df = df.merge(ids_table, left_on='participant', right_index = True, how = 'left')
    # Filtering out participants with no associated redcap data
    no_redcap = df.query("redcap_id.isna()", engine="python").participant.unique()
    print("Participants: %s have no associated redcap IDs and are excluded from the following analyses."%', '.join(no_redcap))
    df = df[df.redcap_id.isna()==False]
    return df

# Cell
@patch
def get_alcohol_per_drink(self:DataProvider):
    if not os.path.isfile(self.alcohol_per_drink_path):
        return None
    else:
        return pd.read_csv(self.alcohol_per_drink_path)

# Cell
@patch
@get_efficiently
def get_two_day_data(self:DataProvider):
    """Gets two-day which are a subset of mov_data
    1) We select two-day forms and turn the dataframe into one row per participant-date (Form repetitions on the same day are removed)
    2) We reduce all drinking questions to an easily readable dictionary of drink amounts.
    3) We add missing data for dates on which we did not receive answers (starting 2 days before starting date up to 365 days after)
    4) We shift answers from retrospective drinking questions backwards to the adequate date (e.g. one day back for yesterday questions)
    """
    mov_data = self.get_mov_data().reset_index()
    drinking_columns = [c for c in mov_data.columns if c.startswith("Anzahl") and "10day" not in c]
    mdbf_columns = [c for c in mov_data.columns if "MDBF" in c and "INT" not in c]
    two_day_columns = ['mov_index','starting_date','end_date','sampling_day'] + drinking_columns + mdbf_columns + ['soziale_isolation','alternative_rewards','craving','Limit','Kontrolle']
    # 1) Turning df into one line per date
    two_day_forms = ["Coverage","Soziale Isolation","Craving","MDBF","Alternative Rewards","Limits & Control"]
    df = mov_data.sort_values(by=['participant','trigger_date','Form','Trigger_counter'])
    df = df[df.Form.isin(two_day_forms)].groupby(["participant","date"])[two_day_columns].first().dropna(how='all').reset_index()
    # 2) Turning drinking answers into dictionaries
    df[drinking_columns].fillna(0,inplace = True)
    df['drinks_gestern'] = df[[c for c in drinking_columns if "vorgestern" not in c]].fillna(0).agg(lambda x: x.to_dict(), axis=1)
    df['drinks_vorgestern'] = df[[c for c in drinking_columns if "vorgestern" in c]].fillna(0).agg(lambda x: x.to_dict(), axis=1)
    # Adding missing values
    def add_missing_data(df):
        dates = pd.date_range(df.starting_date.iloc[0]-pd.DateOffset(days = 2), df.end_date.iloc[0])
        df = df.reindex(dates)
        df.index.names = ['date']
        df[['starting_date','sampling_day']] = df[['starting_date','sampling_day']].bfill().ffill()
        # 3) Shifting back retrospective answers
        df['drinks'] = df['drinks_gestern'].shift(-1) # Drinks yesterday get shifted back one day
        df['drinks'].fillna(df['drinks_vorgestern'].shift(-2), inplace = True) # Drinks before yesterday shift back two days
        df['limit'] = df.Limit.ffill(limit=8) # Limit gets repeated forward for eight days
        df['control'] = df.Kontrolle.bfill(limit=8) # Control is repeated backward for eight days
        return df
    df = df.set_index('date').groupby('participant').apply(add_missing_data).drop(columns='participant').reset_index()


    # 4) Calculating daily alcohol consumption in grams
    alc_dict = {"Anzahl"+k:v for k,v in self.get_alcohol_per_drink().set_index('drink')['ml_alc_per_drink'].to_dict().items()}
    def calculate_g_alc(x):
        if x != x:
            ml_alc = np.nan
        else:
            ml_alc = 0
            for k,v in x.items():
                if v > 0:
                    ml_alc += alc_dict[k.split('_')[0]] * v
        return ml_alc * .8
    df['g_alc']  = df.drinks.apply(calculate_g_alc)
    # Returning relevant columns only
    df = df.reset_index()
    df['sampling_day'] = df.groupby('participant').starting_date.cumcount() + 1
    df.index.rename('two_day_index',inplace = True)
    return df[['mov_index','participant','starting_date','date','sampling_day'] + mdbf_columns + ['soziale_isolation','alternative_rewards','craving','Limit','limit','Kontrolle','control','drinks','g_alc']]


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/09_gbe.ist.data_provider.ipynb (unless otherwise specified).

__all__ = ['ISTDataProvider']

# Cell
from fastcore.foundation import patch
from ..data_provider import GBEProvider
from ...data_provider import get_efficiently
import numpy as np
import os
import pandas as pd

# Cell
class ISTDataProvider(GBEProvider):
    '''This class builds upon GBEProvider to get the working memory task data.'''
    def __init__(self, data_folder_path):
        GBEProvider.__init__(self, data_folder_path)

# Cell
@patch
def get_optimal_turns(self:ISTDataProvider):
    with open(os.path.join(self.external,'opt_string.txt'),'r') as f: #
        optimal_turns = f.read()
    # Getting optimal turns from Hunt's model
    ot = optimal_turns.split('\n')
    ot = [x.split(':') for x in ot][:-1]
    ot = [[list(eval(x[0])),int(x[1])] for x in ot]
    ot = pd.DataFrame(ot)
    ot.columns = ['card_values','optimal_turns']
    ot['deck_id'] = ot.card_values.apply(lambda x: ''.join([str(y-1) for y in x]))
    return ot


# Cell
@patch
def decode_ist_strings(self:ISTDataProvider, gbe_data):
    df = self.decode_gbe_strings(gbe_data, "CardGame")
    back_to_list = lambda x: x.strip('[]').replace('|',',').split(',')
    df['cardlist'] = df.cardlist.apply(back_to_list)
    df['movelist'] = df.movelist.apply(back_to_list)
    df['turntime'] = df.turntime.apply(back_to_list)
    df['deck_id'] = df.cardlist.apply(lambda x: ''.join([str((int(c)-1)%10) for c in x]))
    df['turns'] = df.turntime.apply(lambda x: len(x))
    df['first_card'] = df.deck_id.astype(str).apply(lambda x: int(x[0])+1)
    ot = self.get_optimal_turns()
    df = df.merge(ot[['deck_id','optimal_turns']], on='deck_id',validate='many_to_one').sort_values(by=['gbe_index','trial_number']).reset_index(drop = True)
    df['oversampling'] = df.turns - df.optimal_turns
    return df

# Cell
@patch
@get_efficiently
def get_ist_data(self:ISTDataProvider):
    gbe_data = self.get_gbe_data()
    df = self.decode_ist_strings(gbe_data)
    return df

# Cell
@patch
def filter_old_app_sessions(self:ISTDataProvider, df):
    is_multiplicative = (df.set_index('gbe_index')['trialtype'].astype(int)<2).reset_index().groupby('gbe_index').max()
    sessions_with_old_app = list(is_multiplicative[is_multiplicative.trialtype].index)
    total_sessions = len(df.gbe_index.unique())
    perc_removed = (len(sessions_with_old_app)/total_sessions)*100
    print("In %d (%.2f%%) sessions participants used an old version of the task in some of their sessions.  These sessions  were removed from the dataset."%(len(sessions_with_old_app), perc_removed))
    df = df[df.gbe_index.isin(sessions_with_old_app)==False]
    return df


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/05_stats.ipynb (unless otherwise specified).

__all__ = ['icc_description_dict', 'describe_iccs', 'retest_plot', 'jitter', 'analyze_reliability', 'get_psych_iccs',
           'get_model_calculated_icc', 'var_comp_plot', 'get_icc_and_variance', 'get_scores', 'get_time_effect',
           'plot_time_effect', 'plot_time_effect_area', 'decompose_within_between']

# Cell
from scipy.stats import pearsonr
import matplotlib.pyplot as plt
import random
import os
import pandas as pd
import seaborn as sns
import numpy as np
from IPython.core.display import display, HTML, Image
import biuR.wrapper

# Cell
def icc_description_dict(iccs):
    """A helper function for formatting icc output.
    """
    format_icc = lambda x: ("%.2f"%x)[1:]
    iccs[0]['description'] = iccs[0]['ICC'].apply(format_icc) + ' (' + iccs[0]['lower bound'].apply(format_icc) + ', ' + iccs[0]['upper bound'].apply(format_icc) + ')'
    return {k:v for k,v in zip(iccs[0].type,iccs[0]['description'])}

def describe_iccs(iccs, description = "", silent = False):
    """A helper function for formatting icc output.
    """
    description_dict = icc_description_dict(iccs)
    description_dict['description'] = description
    if not silent:
        display(HTML("The ICC(1) for %(description)s was %(ICC1)s.  The ICC(1,k) for %(description)s was %(ICC1k)s."%description_dict))
        display(iccs[0].round(3))
        display(iccs[1].round(3))
    return description_dict

def retest_plot(df, add_jitter):
    #fig, axs = plt.subplots(1,3, figsize = (16,4))
    fig = plt.figure(figsize = (10,8))
    gs = fig.add_gridspec(3,2)
    axs = []
    axs.append(fig.add_subplot(gs[0, 0]))
    axs.append(fig.add_subplot(gs[0, 1]))
    axs.append(fig.add_subplot(gs[1:, :]))
    for i in range(2):
        df.iloc[:,i].hist(ax = axs[i])
        axs[i].title.set_text(df.columns[i])
        axs[i].grid(False)
    if add_jitter:
        df[str(df.columns[0])+'_j'] = jitter(df.iloc[:,0])
        df[str(df.columns[1])+'_j'] = jitter(df.iloc[:,1])
        ax = df.plot.scatter(x=str(df.columns[0])+'_j',y=str(df.columns[1])+'_j', ax = axs[2], alpha = .8)
        ax.grid(False)
    else:
        ax = df.plot.scatter(x=df.columns[0],y=df.columns[1], ax = axs[2])
        ax.grid(False)
    sns.despine()
    return fig

def jitter(arr):
    return arr + np.random.randn(len(arr)) * arr.std()/10

def analyze_reliability(df, dependent, add_jitter = False, title = '', silent = False):
    df['_session'] = df.session_number.apply(lambda x: "Session %d"%x)
    ssrt_reg = df.set_index(['participant','_session']).unstack()[dependent].dropna()
    iccs = get_psych_iccs(ssrt_reg)
    describe_iccs(iccs,dependent, silent = silent)
    if not silent:
        fig = retest_plot(ssrt_reg, add_jitter)
        title = title if len(title) else "Test-retest reliability: %s"%dependent
        fig.suptitle(title);
        plt.show()
        return {'fig':fig, 'iccs':iccs}
    else:
        return {'fig':None, 'iccs':iccs}


def get_psych_iccs(df):
    """This function calculates ICCs using the psych package.
    """
    R = biuR.wrapper.R()
    R("""
    require(psych)
    options(warn=-1)
    result = psych::ICC(df)
    # Adding the quotes seems to be import because of glitch that leaks python globals into r
    iccs = result$"results"# -o iccs
    lmers = result$"lme"# -o lmers
    """, push = dict(df=df))
    iccs = R("""iccs""")
    lmers = R("""lmers""")

    return iccs,lmers

def get_model_calculated_icc(m):
    R = biuR.wrapper.R()
    p = R("""
    require(insight)
    var_sess <- get_variance_intercept(m)[1]
    var_pp <- get_variance_intercept(m)[2]
    icc <- var_pp / (var_pp+(var_sess))
    """,push=dict(m=m))

    icc = R("""icc""")
    var_sess = R("""var_sess""")
    var_pp = R("""var_pp""")
    variances = pd.DataFrame({"type":["Participant","Session","Residual"],"variance":[var_pp[0],0,var_sess[0]]})
    variances["Percent"] = variances.variance/variances.variance.sum()
    variances = {'iccs':[None, variances]}
    return {'icc':icc[0],'variances':variances}

# Cell
def var_comp_plot(results):
    perc_variance = []
    for r in results.items():
        _df = r[1]['iccs'][1].T
        _df.columns = ['Participant','Session','Residual','Total']
        perc_variance.append(_df[['Participant','Session','Residual']].loc['Percent'])
    df = pd.DataFrame(perc_variance)
    df.index = results.keys()
    ax = df.plot(kind = 'bar', stacked = True)
    ax.grid(False)
    sns.despine()
    ax.legend(bbox_to_anchor=(1.12, 1.05))
    plt.xticks(rotation=0)
    return ax

# Cell
def get_icc_and_variance(_df,
            dependent,
           icc_type = 'ICC2', scaling = 'Percent'):
    '''Helper function that gets ICCs and variances'''
    outcomes = analyze_reliability(_df, dependent = dependent, silent = True)['iccs']
    icc = outcomes[0].query('type=="%s"'%icc_type)['ICC'].iloc[0]
    variances = outcomes[1].T
    variances.columns = ['Participant','Session','Residual','Total']
    variances = variances.loc[scaling]
    return icc,variances.loc['Participant'],variances.loc['Session'],variances.loc['Residual']

def get_scores(dp, data_function, scoring_function, filters = []):
    # Getting the data
    _df = data_function()
    _df = _df.merge(dp.get_gbe_data(columns = ['participant','session_number','is_initial','is_baseline','time_since_last_gbe']), left_on = 'gbe_index', right_index = True, how = 'left')
    # Filtering
    _df = _df[_df.is_initial==True]
    for filt in filters:
        _df = filt(_df)
    # Scoring
    res = scoring_function(_df)
    if len(res) == 2:
        _df, ms = res
    else:
        _df = res

    _df = _df.merge(dp.get_gbe_data(columns = ['participant','session_number','is_initial','is_baseline','time_since_last_gbe']), left_on = 'gbe_index', right_index = True, how = 'left')
    return _df

def get_time_effect(_df, sessions, dependent, scaling):
    time_data = []
    # Checking which participants have all sessions
    pps = _df.query('session_number<%d'%(sessions+1)).groupby('participant')[_df.columns[0]].agg(lambda x: len(x.dropna()))==sessions
    pps = pps[pps].index
    print("N = %d"%(len(pps)))

    for i in range(sessions-1):
        session_filter = "(session_number==1) or (session_number==%d)"%(i+2)
        __df = _df.query(session_filter)
        __df = __df[__df.participant.isin(pps)]
        d = {}
        d['icc'],d['participant'],d['session'],d['residual'] = get_icc_and_variance(__df, dependent, scaling = scaling)
        time_data.append(d)
    df = pd.DataFrame(time_data, index = pd.Series(range(sessions-1)))
    return df


def plot_time_effect(_df, sessions, dependent, legend = False, scaling = 'Percent', change = False):
    df = get_time_effect(_df, sessions, dependent, scaling)
    if change:
        df[['participant','session','residual']] = (df[['participant','session','residual']]/df[['participant','session','residual']].iloc[0])*100
        ax = df[['icc']].plot(legend = legend, color = 'black', linestyle='--')
        ax.set_ylabel('icc')
        ax2 = ax.twinx()
        ax = df[['participant','session','residual']].plot(legend = legend, ax = ax2)
        if legend:
            ax.figure.legend(bbox_to_anchor=[1.4, 1])
        sns.despine(right=False)
        ax2.set_ylabel('% of initial')
    else:
        ax = df.plot(y="participant", legend=False, color = sns.color_palette()[0])
        ax.set_ylabel('participant', color = sns.color_palette()[0])
        ax2 = ax.twinx()
        df.plot(y="session", ax=ax2, legend=False, color = sns.color_palette()[1])
        ax2.set_ylabel('session',color = sns.color_palette()[1])
        ax3 = ax.twinx()
        ax3.set_ylabel('residual',color = sns.color_palette()[2])
        ax3.spines["right"].set_position(("axes", 1.15))
        df.plot(y="residual", ax=ax3, legend=False, color = sns.color_palette()[2])

        sns.despine(right=False)
        ax.ticklabel_format(style='sci',scilimits=(0,0),axis='both')
        ax2.ticklabel_format(style='sci',scilimits=(0,0),axis='both')
        ax2.get_yaxis().get_offset_text().set_position((1,0))
        ax3.ticklabel_format(style='sci',scilimits=(0,0),axis='both')
        ax3.get_yaxis().get_offset_text().set_position((1.15,0))
    ax.xaxis.get_major_locator().set_params(integer=True)
    return ax

def plot_time_effect_area(_df, sessions, dependent, legend = False):
    time_data = []
    # Checking which participants have all sessions
    pps = _df.query('session_number<%d'%(sessions+1)).groupby('participant')[_df.columns[0]].agg(lambda x: len(x.dropna()))==sessions
    pps = pps[pps].index
    print("N = %d"%(len(pps)))

    for i in range(sessions-1):
        session_filter = "(session_number==1) or (session_number==%d)"%(i+2)
        __df = _df.query(session_filter)
        __df = __df[__df.participant.isin(pps)]
        d = {}
        d['icc'],d['participant'],d['session'],d['residual'] = get_icc_and_variance(__df, dependent)
        time_data.append(d)
    df = pd.DataFrame(time_data, index = pd.Series(range(sessions-1)))
    ax = df[['participant','session','residual']].plot.area(legend = legend)
    df['icc'].plot(color = 'white', linestyle = '--')
    ax.xaxis.get_major_locator().set_params(integer=True)

# Cell
def decompose_within_between(df, columns=None, groupby='participant', result='both', drop = True):
    if not columns:
        columns = list(df.columns.drop(groupby))
    # New column names
    within_columns = [c+'_within' for c in columns]
    between_columns = [c+'_between' for c in columns]
    # Calculating cluster means
    cluster_means = df.groupby('participant')[columns].transform('mean')
    # Subtracting cluster means from initial data to get within effects
    if (result=='both') or (result=='within'):
        df[within_columns] = df[columns] - cluster_means
    # Subtracting cluster means from initial data to get within effects
    if (result=='both') or (result=='between'):
        df[between_columns] = cluster_means - df[columns].mean()
    if drop:
        df = df.drop(columns = columns)
    return df


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/04_gbe.wm.scoring.ipynb (unless otherwise specified).

__all__ = ['get_max_levels', 'get_perc_correct', 'add_implied_data', 'get_implied_perc_correct',
           'get_perc_correct_predicted_sep_r', 'get_perc_correct_predicted_sep',
           'get_perc_correct_predicted_sep_trial_r', 'get_perc_correct_predicted_sep_trial']

# Cell
from .data_provider import WMDataProvider
import pandas as pd
import numpy as np
from scipy import stats
import biuR.wrapper

# Cell
def get_max_levels(df):
    '''This function calculates max levels from wm data.'''
    # Selecting successful trials only.
    successful_trials = df.query('success==1')
    # Grouping by participant, session, and condition and getting the maximum level.
    max_levels = successful_trials.groupby(['gbe_index','trial_type'])['level'].max().unstack()
    # Labeling variables
    max_levels = max_levels[['no_distractor_1', 'no_distractor_2', 'encoding_distractor', 'delayed_distractor']]
    max_levels = max_levels.add_prefix("max_level_")
    return max_levels

# Cell
def get_perc_correct(df):
    '''This function calculates percentage correct from wm data.'''
    max_levels = df.groupby(['gbe_index','trial_type'])['success'].mean().unstack()
    max_levels = max_levels[['no_distractor_1', 'no_distractor_2', 'encoding_distractor', 'delayed_distractor']]
    max_levels = max_levels.add_prefix("perc_correct_")
    return max_levels

# Cell
def add_implied_data(df):
    # Checking whether participants needed one or two trials to complete a session
    df['duplication'] = df.groupby(['participant','session_number','trial_type','level']).cumcount()
    _df = df.set_index(['participant','session_number','trial_type','level','duplication']).sort_index()[['success']]
    # Imputing data
    levels = list(range(3,11))
    new_index = pd.MultiIndex.from_product([_df.reset_index().participant.unique(), _df.reset_index().session_number.unique(),_df.reset_index().trial_type.unique(), levels, [0,1]],names = ['participant','session_number','trial_type','level','duplication'])
    _df = _df.reindex(new_index)
    _df['implied_success'] = _df.groupby(level=['participant','session_number','trial_type','level']).transform(np.max).fillna(0)
    _df['implied_success'] = _df.success.fillna(_df.implied_success)
    _df = _df.reset_index()
    #_df['implied_success'] = _df.implied_success# * _df.level
    _df = _df.merge(df.groupby(['participant','session_number']).gbe_index.first().reset_index(), on = ['participant','session_number'], how = 'outer' )
    _df = _df[_df.gbe_index.isnull()==False]
    #implied_success = _df.groupby(['gbe_index']).implied_success.mean()
    return _df#, implied_success

# Cell
def get_implied_perc_correct(df):
    df = add_implied_data(df)
    '''This function calculates percentage correct from wm data.'''
    implied_perc_correct = df.groupby(['gbe_index','trial_type'])['implied_success'].mean().unstack()
    implied_perc_correct = implied_perc_correct[['no_distractor_1', 'no_distractor_2', 'encoding_distractor', 'delayed_distractor']]
    implied_perc_correct = implied_perc_correct.add_prefix("impl_perc_correct_")
    return implied_perc_correct

# Cell
#def get_perc_correct_predicted_sep_r(df):
#    %R -i df
#    %R library(lmerTest)
#    %R library(ggeffects)
#    # Running the model
#    %R control=glmerControl(optimizer = "bobyqa", optCtrl=list(maxfun=1e6))
#    %R m = glmer(implied_success ~ 1 + (1 | participant), data=df, family=binomial, control = control, na.action = na.exclude)
#    # Extracting predicted values
#    %R p = ggpredict(m, terms=c("participant"), type="re",ci.lvl = NA)
#    %R -o p
#    return p

def get_perc_correct_predicted_sep_r(df):
    R = biuR.wrapper.R()
    p = R("""
    library(lmerTest)
    library(ggeffects)
    # Running the model
    control=glmerControl(optimizer = "bobyqa", optCtrl=list(maxfun=1e6))
    m = glmer(implied_success ~ 1 + (1 | participant), data=df, family=binomial, control = control, na.action = na.exclude)
    # Extracting predicted values
    ggpredict(m, terms=c("participant"), type="re",ci.lvl = NA)
    """,push=dict(df=df))
    return p

def get_perc_correct_predicted_sep(df):
    df = add_implied_data(df)
    df = df[['implied_success','participant','session_number','trial_type']]

    dfs = []
    # Looping through trial types
    for trial_type in ['no_distractor_2','no_distractor_1','encoding_distractor','delayed_distractor']:
        session_dfs = []
        # Looping through sessions
        for session in [2,1]:
            # Extracting data for specific session and trial type
            _df = df.query('(session_number==@session) and (trial_type==@trial_type)')[['implied_success','participant']]
            # Predicting scores
            predicted = get_perc_correct_predicted_sep_r(_df)
            # Labeling variables
            predicted.columns = ['participant','perc_predicted_sep_%s'%trial_type,'session']
            predicted['session'] = session
            predicted['gbe_index'] = predicted.participant.astype(str) + '_%03d'%session
            predicted = predicted.set_index('gbe_index')['perc_predicted_sep_%s'%trial_type].to_frame()
    # Combining everything into one dataframe
            session_dfs.append(predicted)
        dfs.append(pd.concat(session_dfs))
    perc_predicted_sep = pd.concat(dfs, axis = 1)
    return perc_predicted_sep

# Cell
def get_perc_correct_predicted_sep_trial_r(df):
    R = biuR.wrapper.R()
    p = R("""
    library(lmerTest)
    library(ggeffects)
    # Running the model
    control=glmerControl(optimizer = "bobyqa", optCtrl=list(maxfun=1e6))
    m = glmer(implied_success ~ 1 + (1 | participant/session), data=df, family=binomial, control = control, na.action = na.exclude)
    # Extracting predicted values
    ggpredict(m, terms=c("participant","session"), type="re",ci.lvl = NA)
    """,push=dict(df=df))

    m = R("""m""")
    return p, m

def get_perc_correct_predicted_sep_trial(df):
    df = add_implied_data(df)
    dfs = []
    ms = {}
    # Looping through trial types
    for trial_type in ['no_distractor_2','no_distractor_1','encoding_distractor','delayed_distractor']:
        # Extracting data for specific trial type
        _df = df.query('(trial_type==@trial_type)')[['implied_success','participant','session_number']]
        _df['session'] = _df.session_number.astype(str) # making session a factor
        # Predicting scores
        predicted, m = get_perc_correct_predicted_sep_trial_r(_df)
        # Labeling variables
        predicted.columns = ['participant','perc_predicted_sep_trial_%s'%trial_type,'session']
        #predicted['session'] = session
        predicted['gbe_index'] = predicted.participant.astype(str) + predicted.session.apply(lambda x: '_%03d'%int(float(x))).astype(str)
        predicted = predicted.set_index('gbe_index')['perc_predicted_sep_trial_%s'%trial_type].to_frame()
    # Combining everything into one dataframe
        dfs.append(predicted)
        ms[trial_type] = m
    perc_predicted_sep_trial = pd.concat(dfs, axis = 1)
    # Removing sessions that were not in initial dataframe
    perc_predicted_sep_trial = perc_predicted_sep_trial.loc[df.gbe_index.unique()]
    return perc_predicted_sep_trial, ms



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/06_gbe.wm.reliability.ipynb (unless otherwise specified).

__all__ = []

# Cell
%load_ext autoreload
%autoreload 2
from .data_provider import WMDataProvider
import trr265.gbe.wm.scoring as wm_scoring
import trr265.stats as stats
import pandas as pd


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/25_gbe.ema_drinking_checkpoint.ipynb (unless otherwise specified).

__all__ = ['pearson_r', 'lmer', 'cor', 'pearson_r', 'lmer', 'cor', 'pearson_r', 'lmer', 'cor', 'pearson_r', 'lmer',
           'cor']

# Cell
%load_ext autoreload
%autoreload 2
from .ist.data_provider import ISTDataProvider
from .wm.data_provider import WMDataProvider
from .sst.data_provider import SSTDataProvider
from .rtt.data_provider import RTTDataProvider

import trr265.gbe.ist.scoring as ist_scoring
import trr265.gbe.wm.scoring as wm_scoring
import trr265.gbe.sst.scoring as sst_scoring
import trr265.gbe.rtt.scoring as rtt_scoring

import pandas as pd

# Cell
cor = pd.concat([drinking, tasks], axis = 1).dropna()
import biuR

def pearson_r(x, y, df):
    return scipy.stats.pearsonr(df[x], df[y])


#export
def lmer(x, y, df):
    R = biuR.wrapper.R()
    p = R("""
    library(lmerTest)
    library(ggeffects)
    # Running the model
    control=lmerControl(optimizer = "bobyqa", optCtrl=list(maxfun=1e6))
    m = lmer(%(y)s ~ %(x)s + (1 | participant), data=df, na.action = na.exclude)
    estimate = summary(m)$coefficients[2]
    p_value = summary(m)$coefficients[10]
    """%{'x':x,'y':y},push=dict(df=df))


    m = R("""m""")
    estimate = R("""estimate""")
    p_value = R("""p_value""")

    return estimate[0], p_value[0]

lmer('wm_no_1','g_alc_mean',cor)

# Cell
%load_ext autoreload
%autoreload 2
from .ist.data_provider import ISTDataProvider
from .wm.data_provider import WMDataProvider
from .sst.data_provider import SSTDataProvider
from .rtt.data_provider import RTTDataProvider

import trr265.gbe.ist.scoring as ist_scoring
import trr265.gbe.wm.scoring as wm_scoring
import trr265.gbe.sst.scoring as sst_scoring
import trr265.gbe.rtt.scoring as rtt_scoring

import pandas as pd

# Cell
cor = pd.concat([drinking, tasks], axis = 1).dropna()
import biuR

def pearson_r(x, y, df):
    return scipy.stats.pearsonr(df[x], df[y])


#export
def lmer(x, y, df):
    R = biuR.wrapper.R()
    p = R("""
    library(lmerTest)
    library(ggeffects)
    # Running the model
    control=lmerControl(optimizer = "bobyqa", optCtrl=list(maxfun=1e6))
    m = lmer(%(y)s ~ %(x)s + (1 | participant), data=df, na.action = na.exclude)
    estimate = summary(m)$coefficients[2]
    p_value = summary(m)$coefficients[10]
    """%{'x':x,'y':y},push=dict(df=df))


    m = R("""m""")
    estimate = R("""estimate""")
    p_value = R("""p_value""")

    return estimate[0], p_value[0]

lmer('wm_no_1','g_alc_mean',cor)

# Cell
%load_ext autoreload
%autoreload 2
from .ist.data_provider import ISTDataProvider
from .wm.data_provider import WMDataProvider
from .sst.data_provider import SSTDataProvider
from .rtt.data_provider import RTTDataProvider

import trr265.gbe.ist.scoring as ist_scoring
import trr265.gbe.wm.scoring as wm_scoring
import trr265.gbe.sst.scoring as sst_scoring
import trr265.gbe.rtt.scoring as rtt_scoring

import pandas as pd

# Cell
cor = pd.concat([drinking, tasks], axis = 1).dropna()
import biuR

def pearson_r(x, y, df):
    return scipy.stats.pearsonr(df[x], df[y])


#export
def lmer(x, y, df):
    R = biuR.wrapper.R()
    p = R("""
    library(lmerTest)
    library(ggeffects)
    # Running the model
    control=lmerControl(optimizer = "bobyqa", optCtrl=list(maxfun=1e6))
    m = lmer(%(y)s ~ %(x)s + (1 | participant), data=df, na.action = na.exclude)
    estimate = summary(m)$coefficients[2]
    p_value = summary(m)$coefficients[10]
    """%{'x':x,'y':y},push=dict(df=df))


    m = R("""m""")
    estimate = R("""estimate""")
    p_value = R("""p_value""")

    return estimate[0], p_value[0]

lmer('wm_no_1','g_alc_mean',cor)

# Cell
%load_ext autoreload
%autoreload 2
from .ist.data_provider import ISTDataProvider
from .wm.data_provider import WMDataProvider
from .sst.data_provider import SSTDataProvider
from .rtt.data_provider import RTTDataProvider

import trr265.gbe.ist.scoring as ist_scoring
import trr265.gbe.wm.scoring as wm_scoring
import trr265.gbe.sst.scoring as sst_scoring
import trr265.gbe.rtt.scoring as rtt_scoring

import pandas as pd

# Cell
cor = pd.concat([drinking, tasks], axis = 1).dropna()
import biuR

def pearson_r(x, y, df):
    return scipy.stats.pearsonr(df[x], df[y])


#export
def lmer(x, y, df):
    R = biuR.wrapper.R()
    p = R("""
    library(lmerTest)
    library(ggeffects)
    # Running the model
    control=lmerControl(optimizer = "bobyqa", optCtrl=list(maxfun=1e6))
    m = lmer(%(y)s ~ %(x)s + (1 | participant), data=df, na.action = na.exclude)
    estimate = summary(m)$coefficients[2]
    p_value = summary(m)$coefficients[10]
    """%{'x':x,'y':y},push=dict(df=df))


    m = R("""m""")
    estimate = R("""estimate""")
    p_value = R("""p_value""")

    return estimate[0], p_value[0]

lmer('wm_no_1','g_alc_mean',cor)


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/03_gbe.wm.data_provider.ipynb (unless otherwise specified).

__all__ = ['WMDataProvider']

# Cell
import os
from bs4 import BeautifulSoup
from fastcore.foundation import patch
from ..data_provider import GBEProvider
from ...data_provider import get_efficiently
import pandas as pd
import numpy as np
import xmltodict
import collections

# Cell
class WMDataProvider(GBEProvider):
    '''This class builds upon GBEProvider to get the working memory task data.'''
    def __init__(self, data_folder_path):
        GBEProvider.__init__(self, data_folder_path)

# Cell
@patch
def decode_wm_strings(self:WMDataProvider, gbe_data):
    df = self.decode_gbe_strings(gbe_data, 'WorkingMemoryGame')
    df['success'] = df.success.astype(int)
    return df

# Cell
@patch
@get_efficiently
def get_wm_trial_types(self:WMDataProvider):
    '''This checks that each trial type follows the appropriate specifications.'''
    types = open(os.path.join(self.external, "types.xml"),"r", encoding='UTF-8').read().encode('utf-8')
    types_dict = xmltodict.parse(types)
    type_list = types_dict['trials']['trialtype']
    type_df = []
    for i, t in enumerate(type_list):
        # Getting the type and difficulty
        trial_type = t['@type']
        level = int(t['@difficulty'])
        if level > 10:
            continue
        # Geting individual trial specifications for each type
        for trial in t['trial']:
            trial_dict = {}
            trial_dict['trialid'] = trial['@id'] #id
            trial_dict['trial_type'] = trial_type #
            trial_dict['level'] = level

            # Getting additional information

            board = trial['board']
            # Checking if circles were shown delayed
            if type(board) == collections.OrderedDict:
                board = [board]
            trial_dict['trial_boards'] = []
            # Checking if the trial had distractors
            has_distractor = False
            trial_dict['number_of_boards'] = 0
            for b in board:
                try:
                    if '#text' in b.keys():
                        trial_dict['trial_boards'].append(b['#text'])
                        trial_dict['number_of_boards'] += 1
                    if '#text' in b.keys() and 'D' in b['#text']:
                        has_distractor = True
                        break
                except:
                    print(trial_dict, board)
            is_delayed = int(trial_dict['number_of_boards'] > 1)
            trial_dict['has_distractor'] = int(has_distractor)
            trial_dict['is_delayed'] = int(trial_dict['number_of_boards'] > 1)
            type_df.append(pd.Series(trial_dict))
    type_df = pd.DataFrame(type_df)
    type_df.trial_type.replace({'0':'no_distractor_1',
                               '1':'encoding_distractor',
                               '2':'delayed_distractor',
                               '3':'no_distractor_2'}, inplace = True)
    return type_df

# Cell
@patch
def add_trial_types(self:WMDataProvider, df):
    type_df = self.get_wm_trial_types()
    if 'trial_type' in df.columns:
        df = df.drop.columns('trial_type')
    df = df.merge(type_df[['trialid','trial_type','level']], on = 'trialid', how = 'left', validate = 'many_to_one')
    return df


# Cell
@patch
@get_efficiently
def get_wm_data(self:WMDataProvider):
    gbe_data = self.get_gbe_data()
    df = self.decode_wm_strings(gbe_data)
    df = self.add_trial_types(df)
    return df

# Cell
@patch
def filter_old_app_sessions(self:WMDataProvider, df):
    participants_with_old_app = self.get_gbe_data().loc[df[df.trial_type.isin(['4','5'])].gbe_index.unique()].participant.unique()
    sessions_with_old_app = df[df.trial_type.isin(['4','5'])].gbe_index.unique()
    total_sessions = len(df.gbe_index.unique())
    perc_removed = (len(sessions_with_old_app)/total_sessions)*100
    print("%d participants used an old version of the task in some of their sessions.  %d sessions (%.2f%%) were removed from the dataset."%(len(participants_with_old_app), len(sessions_with_old_app), perc_removed))
    df = df[df.gbe_index.isin(sessions_with_old_app)==False]
    return df

# Cell
@patch
def filter_level_two_failures(self:WMDataProvider, df):
    filtered_sessions = df.query("(level==2) and (success==0)").gbe_index.unique()
    total_sessions = len(df.gbe_index.unique())
    perc_removed = (len(filtered_sessions)/total_sessions)*100
    print("%d sessions (%.2f%%) were removed because participants failed a level two trial."%(len(filtered_sessions), perc_removed))
    df = df[df.gbe_index.isin(filtered_sessions)==False]
    return df


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/24_gbe.rtt.intense_sampling.ipynb (unless otherwise specified).

__all__ = ['get_intense_drinking_data']

# Cell
from .sst.data_provider import SSTDataProvider
import pandas as pd
import numpy as np
from scipy import stats
import biuR.wrapper

# Cell
def get_intense_drinking_data():

    change_dict = {'AnzahlKleinesBier': 'kleinesBier',
 'AnzahlMittlereBier': 'mittleresBier',
 'AnzahlGrosseBier': 'grossesBier',
 'AnzahlKleinerWeisswein': 'kleinerWeisswein',
 'AnzahlMittlererWeisswein': 'mittlererWeisswein',
 'AnzahlFlascheWeisswein': 'FlascheWeisswein',
 'AnzahlKleinerRotwein': 'kleinerRotwein',
 'AnzahlMittlererRotwein': 'mittlereRotwein',
 'AnzahlFlaschenRotwein': 'FlascheRotwein',
 'AnzahlSekt': 'Sekt',
 'AnzahlFlaschenSekt': 'FlascheSekt',
 'AnzahlLikoerWein': 'Likoer-Wein',
 'AnzahlKleineLikoer': 'kleinerLikoer',
 'AnzahlGrosseLikoer': 'grosserLikoer',
 'AnzahlLikoerSuess': 'LikoerSuess',
 'AnzahlKleineSpirituosen': 'kleineSpirituosen',
 'AnzahlGrosseSpirituosen': 'grosseSpirituosen',
 'AnzahlSpirituosen100ml': 'Spirituose',
 'AnzahlSpirituosenStark': 'SpirituoseStark'}

    alc_dict = {"Anzahl"+k:v for k,v in dp.get_alcohol_per_drink().set_index('drink')['ml_alc_per_drink'].to_dict().items()}
    alc_dict_1 = dict(("INT_"+change_dict[key], value) for (key, value) in alc_dict.items())
    alc_dict_1['INT_Weinbrand'] = 7.2

    alc_dict_2 = dict(("INT_Morgenabfr_"+change_dict[key]+'_gesternAbend', value) for (key, value) in alc_dict.items())
    alc_dict_2['INT_Morgenabfr_Weinbrand_gesternAbend'] = 7.2
    alc_dict = {**alc_dict_1, **alc_dict_2}

    change_dict2 = {
'INT_Morgenabfr_kleinerLikoer_gesternAbend':'INT_Morgenabfr_Likoerklein_gesternAbend',
 'INT_Morgenabfr_kleineSpirituosen_gesternAbend':'INT_Morgenabfr_kleineSpirituose_gesternAbend',
 'INT_Morgenabfr_grosseSpirituosen_gesternAbend':'INT_Morgenabfr_grosseSpirituose_gesternAbend',
 'INT_Morgenabfr_Likoer-Wein_gesternAbend':'INT_Morgenabfr_LikoerWein_gesternAbend',
 'INT_Morgenabfr_LikoerSuess_gesternAbend':'INT_Morgenabfr_Likoersuess_gesternAbend',
 'INT_Morgenabfr_SpirituoseStark_gesternAbend':'INT_Morgenabfr_kleineSpirituoseStark_gesternAbend',
 'INT_Morgenabfr_mittlereRotwein_gesternAbend':'INT_Morgenabfr_mittlererRotwein_gesternAbend',
 'INT_Morgenabfr_grosserLikoer_gesternAbend':'INT_Morgenabfr_Likoergross_gesternAbend'}

    alc_dict = dict((change_dict2[key] if key in change_dict2 else key, value) for (key, value) in alc_dict.items())

    mov_data = dp.get_mov_data().reset_index()
    drinking_columns = list(alc_dict.keys())#[c for c in mov_data.columns if c.startswith("INT_")]
    two_day_columns = ['mov_index','starting_date','end_date','sampling_day'] + drinking_columns
    # 1) Turning df into one line per date
    two_day_forms = ["Intense_Morgenabfrage","Intense_Tagesabfrage"]
    df = mov_data.sort_values(by=['participant','trigger_date','Form','Trigger_counter'])
    df = df[df.Form.isin(two_day_forms)].groupby(["participant","date"])[two_day_columns].first().dropna(how='all').reset_index()
    # 2) Turning drinking answers into dictionaries
    df[drinking_columns].fillna(0,inplace = True)
    df['drinks_heute'] = df[[c for c in drinking_columns if "gestern" not in c]].fillna(0).agg(lambda x: x.to_dict(), axis=1)
    df['drinks_gestern'] = df[[c for c in drinking_columns if "gestern" in c]].fillna(0).agg(lambda x: x.to_dict(), axis=1)
    # Adding missing values

    # TODO
    #def add_missing_data(df):
    #    dates = pd.date_range(df.starting_date.iloc[0]-pd.DateOffset(days = 2), df.end_date.iloc[0])
    #    df = df.reindex(dates)
    #    df.index.names = ['date']
    #    df[['starting_date','sampling_day']] = df[['starting_date','sampling_day']].bfill().ffill()
    #    return df
    #df = df.set_index('date').groupby('participant').apply(add_missing_data).drop(columns='participant').reset_index()

    # 3) Shifting back retrospective answers
    df['drinks_1'] = df['drinks_heute']
    df['drinks_2'] = df['drinks_gestern'].shift(-1) # Drinks yesterday shift back one days
    def combine_dicts(drinks_1, drinks_2):
        if type(drinks_2)!=dict:
            return drinks_1
        else:
            d = {**drinks_1, **drinks_2}


        return d
    df['drinks'] = df.apply(lambda x: combine_dicts(x.drinks_1, x.drinks_2), axis=1)

    # 4) Calculating daily alcohol consumption in grams
    #alc_dict = {"Anzahl"+k:v for k,v in self.get_alcohol_per_drink().set_index('drink')['ml_alc_per_drink'].to_dict().items()}
    def calculate_g_alc(x):
        if x != x:
            ml_alc = np.nan
        else:
            ml_alc = 0
            for k,v in x.items():
                if v > 0:
                    ml_alc += alc_dict[k]*v#.split('_')[0]] * v
        return ml_alc * .8
    df['g_alc']  = df.drinks.apply(calculate_g_alc)
    # Returning relevant columns only
    df = df.reset_index()
    df['sampling_day'] = df.groupby('participant').starting_date.cumcount() + 1
    df.index.rename('intense_index',inplace = True)
    return df[['mov_index','participant','starting_date','date','sampling_day','g_alc']]
_df = get_intense_drinking_data()


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/12_gbe.ist.scoring.ipynb (unless otherwise specified).

__all__ = ['get_average_oversampling', 'get_oversampling_predicted_sep_r', 'get_oversampling_sep',
           'get_oversampling_predicted_joint_r', 'get_oversampling_predicted_joint']

# Cell
from .data_provider import ISTDataProvider
import pandas as pd
import numpy as np
from scipy import stats
import biuR.wrapper

# Cell
def get_average_oversampling(df):
    return df.groupby(['gbe_index']).oversampling.mean().to_frame()

# Cell
def get_oversampling_predicted_sep_r(df):
    R = biuR.wrapper.R()
    p = R("""
    library(lmerTest)
    library(ggeffects)
    # Running the model
    control=lmerControl(optimizer = "bobyqa", optCtrl=list(maxfun=1e6))
    m = lmer(oversampling ~ 1 + (1 | participant), data=df,  na.action = na.exclude)
    # Extracting predicted values
    ggpredict(m, terms=c("participant"), type="re",ci.lvl = NA)
    """,push=dict(df=df))
    m = R("""m""")
    return p, m

def get_oversampling_sep(df):
    # Creating combined variable for ssd and rt
    dfs = []
    # Looping through sessions
    for session in [2,1]:
        # Extracting data for specific session and trial type
        _df = df.query('(session_number==@session)')
        # Predicting scores
        predicted, m = get_oversampling_predicted_sep_r(_df[['participant','oversampling']])
        predicted.columns = ['participant','oversampling_predicted_sep','session']
        # Labeling variables
        predicted['session'] = session
        predicted['gbe_index'] = predicted.participant.astype(str) + '_%03d'%session
        predicted = predicted.set_index('gbe_index')['oversampling_predicted_sep'].to_frame()
# Combining everything into one dataframe
        dfs.append(predicted)
    perc_predicted_sep = pd.concat(dfs)
    return perc_predicted_sep, m


# Cell
def get_oversampling_predicted_joint_r(df):
    R = biuR.wrapper.R()
    p = R("""
    library(lmerTest)
    library(ggeffects)
    # Running the model
    control=lmerControl(optimizer = "bobyqa", optCtrl=list(maxfun=1e6))
    m = lmer(oversampling ~ 1 + (1 | participant / session), data=df, na.action = na.exclude)
    # Extracting predicted values
    ggpredict(m, terms=c("participant", "session"), type="re",ci.lvl = NA)
    """,push=dict(df=df))

    m = R("""m""")
    return p, m

def get_oversampling_predicted_joint(df):
    df['session'] = df.session_number.astype(str) # making session a factor
    # Predicting scores
    predicted, m = get_oversampling_predicted_joint_r(df[['participant','session','oversampling']])
    predicted.columns = ['participant','oversampling_predicted_joint','session']
    #predicted = predicted[('ssrt_predicted_joint','')].to_frame(name="ssrt_predicted_joint").reset_index()
    predicted['gbe_index'] = predicted.participant.astype(str) + predicted.session.apply(lambda x: '_%03d'%int(float(x))).astype(str)
    predicted = predicted.set_index('gbe_index')['oversampling_predicted_joint'].to_frame()
    # Removing sessions that were not in initial dataframe
    predicted = predicted.loc[df.gbe_index.unique()]
    return predicted, m



# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"DataProvider": "01_data_provider.ipynb",
         "get_efficiently": "01_data_provider.ipynb",
         "DataProvider.store_interim": "01_data_provider.ipynb",
         "DataProvider.load_interim": "01_data_provider.ipynb",
         "DataProvider.get_phone_codebook": "01_data_provider.ipynb",
         "DataProvider.determine_phone_b07": "01_data_provider.ipynb",
         "DataProvider.check_participant_id": "01_data_provider.ipynb",
         "DataProvider.test_check_participant_id": "01_data_provider.ipynb",
         "DataProvider.set_dtypes": "01_data_provider.ipynb",
         "DataProvider.get_phone_data": "01_data_provider.ipynb",
         "DataProvider.get_ba_codebook": "01_data_provider.ipynb",
         "DataProvider.get_ba_data": "01_data_provider.ipynb",
         "DataProvider.get_baseline_drinking_data": "01_data_provider.ipynb",
         "DataProvider.get_duplicate_mov_ids": "01_data_provider.ipynb",
         "DataProvider.get_mov_data": "01_data_provider.ipynb",
         "DataProvider.get_alcohol_per_drink": "01_data_provider.ipynb",
         "DataProvider.get_two_day_data": "01_data_provider.ipynb",
         "GBEProvider": "02_gbe.data_provider.ipynb",
         "GBEProvider.get_gbe_from_movisense": "02_gbe.data_provider.ipynb",
         "GBEProvider.define_initial_dataset": "02_gbe.data_provider.ipynb",
         "GBEProvider.define_baseline_sessions": "02_gbe.data_provider.ipynb",
         "GBEProvider.find_gbe_game_starting_time": "02_gbe.data_provider.ipynb",
         "GBEProvider.check_time_between_sessions": "02_gbe.data_provider.ipynb",
         "determine_ema_session_numbers": "02_gbe.data_provider.ipynb",
         "check_repeating_sessions": "02_gbe.data_provider.ipynb",
         "increase_session_numbers": "02_gbe.data_provider.ipynb",
         "filter_repeating_sessions": "02_gbe.data_provider.ipynb",
         "GBEProvider.fix_repeating_ema_sessions": "02_gbe.data_provider.ipynb",
         "GBEProvider.get_gbe_data": "02_gbe.data_provider.ipynb",
         "GBEProvider.get_initial_baseline": "02_gbe.data_provider.ipynb",
         "GBEProvider.get_replication_baseline": "02_gbe.data_provider.ipynb",
         "GBEProvider.decode_gbe_string": "02_gbe.data_provider.ipynb",
         "GBEProvider.decode_gbe_strings": "02_gbe.data_provider.ipynb",
         "WMDataProvider": "03_gbe.wm.data_provider.ipynb",
         "WMDataProvider.decode_wm_strings": "03_gbe.wm.data_provider.ipynb",
         "WMDataProvider.get_wm_trial_types": "03_gbe.wm.data_provider.ipynb",
         "WMDataProvider.add_trial_types": "03_gbe.wm.data_provider.ipynb",
         "WMDataProvider.get_wm_data": "03_gbe.wm.data_provider.ipynb",
         "WMDataProvider.filter_old_app_sessions": "03_gbe.wm.data_provider.ipynb",
         "WMDataProvider.filter_level_two_failures": "03_gbe.wm.data_provider.ipynb",
         "get_max_levels": "04_gbe.wm.scoring.ipynb",
         "get_perc_correct": "04_gbe.wm.scoring.ipynb",
         "add_implied_data": "04_gbe.wm.scoring.ipynb",
         "get_implied_perc_correct": "04_gbe.wm.scoring.ipynb",
         "get_perc_correct_predicted_sep_r": "04_gbe.wm.scoring.ipynb",
         "get_perc_correct_predicted_sep": "04_gbe.wm.scoring.ipynb",
         "get_perc_correct_predicted_sep_trial_r": "04_gbe.wm.scoring.ipynb",
         "get_perc_correct_predicted_sep_trial": "04_gbe.wm.scoring.ipynb",
         "icc_description_dict": "05_stats.ipynb",
         "describe_iccs": "05_stats.ipynb",
         "retest_plot": "05_stats.ipynb",
         "jitter": "05_stats.ipynb",
         "analyze_reliability": "05_stats.ipynb",
         "get_psych_iccs": "05_stats.ipynb",
         "get_model_calculated_icc": "05_stats.ipynb",
         "var_comp_plot": "05_stats.ipynb",
         "get_icc_and_variance": "05_stats.ipynb",
         "get_scores": "05_stats.ipynb",
         "get_time_effect": "05_stats.ipynb",
         "plot_time_effect": "05_stats.ipynb",
         "plot_time_effect_area": "05_stats.ipynb",
         "decompose_within_between": "05_stats.ipynb",
         "SSTDataProvider": "07_gbe.sst.data_provider.ipynb",
         "SSTDataProvider.decode_sst_strings": "07_gbe.sst.data_provider.ipynb",
         "SSTDataProvider.get_sst_data": "07_gbe.sst.data_provider.ipynb",
         "RTTDataProvider": "08_gbe.rtt.data_provider.ipynb",
         "RTTDataProvider.decode_rtt_strings": "08_gbe.rtt.data_provider.ipynb",
         "RTTDataProvider.get_rtt_data": "08_gbe.rtt.data_provider.ipynb",
         "ISTDataProvider": "09_gbe.ist.data_provider.ipynb",
         "ISTDataProvider.get_optimal_turns": "09_gbe.ist.data_provider.ipynb",
         "ISTDataProvider.decode_ist_strings": "09_gbe.ist.data_provider.ipynb",
         "ISTDataProvider.get_ist_data": "09_gbe.ist.data_provider.ipynb",
         "ISTDataProvider.filter_old_app_sessions": "09_gbe.ist.data_provider.ipynb",
         "integration_methods_wrapper": "10_gbe.sst.scoring.ipynb",
         "verbruggen_quantile": "10_gbe.sst.scoring.ipynb",
         "integration_without_replacement": "10_gbe.sst.scoring.ipynb",
         "get_integration_without_replacement_ssrts": "10_gbe.sst.scoring.ipynb",
         "mean_method_ssrt": "10_gbe.sst.scoring.ipynb",
         "get_mean_method_ssrts": "10_gbe.sst.scoring.ipynb",
         "get_ssrt_predicted_sep_r": "10_gbe.sst.scoring.ipynb",
         "get_ssrt_sep": "10_gbe.sst.scoring.ipynb",
         "get_ssrt_predicted_joint_r": "10_gbe.sst.scoring.ipynb",
         "get_ssrt_predicted_joint": "10_gbe.sst.scoring.ipynb",
         "get_percentage_gamble": "11_gbe.rtt.scoring.ipynb",
         "get_perc_gamble_predicted_sep_r": "11_gbe.rtt.scoring.ipynb",
         "get_perc_gamble_predicted_sep": "11_gbe.rtt.scoring.ipynb",
         "get_perc_gamble_predicted_joint_r": "11_gbe.rtt.scoring.ipynb",
         "get_perc_gamble_predicted_joint": "11_gbe.rtt.scoring.ipynb",
         "get_average_oversampling": "12_gbe.ist.scoring.ipynb",
         "get_oversampling_predicted_sep_r": "12_gbe.ist.scoring.ipynb",
         "get_oversampling_sep": "12_gbe.ist.scoring.ipynb",
         "get_oversampling_predicted_joint_r": "12_gbe.ist.scoring.ipynb",
         "get_oversampling_predicted_joint": "12_gbe.ist.scoring.ipynb",
         "custom_params": "20_plotting.ipynb",
         "icc_comparison_plot": "20_plotting.ipynb",
         "get_intense_drinking_data": "24_gbe.rtt.intense_sampling.ipynb",
         "pearson_r": "25_gbe.ema_drinking_checkpoint.ipynb",
         "lmer": "25_gbe.ema_drinking_checkpoint.ipynb",
         "cor": "25_gbe.ema_drinking_checkpoint.ipynb"}

modules = ["data_provider.py",
           "gbe/data_provider.py",
           "gbe/wm/data_provider.py",
           "gbe/wm/scoring.py",
           "stats.py",
           "gbe/wm/reliability.py",
           "gbe/sst/data_provider.py",
           "gbe/rtt/data_provider.py",
           "gbe/ist/data_provider.py",
           "gbe/sst/scoring.py",
           "gbe/rtt/scoring.py",
           "gbe/ist/scoring.py",
           "gbe/rtt/reliability.py",
           "gbe/sst/reliability.py",
           "gbe/ist/reliability.py",
           "gbe/baseline_drinking.py",
           "gbe/participants.py",
           "plotting.py",
           "validation_task.py",
           "gbe/rtt/singular_fit_inspection.py",
           "gbe/baseline_drinking/separate.py",
           "gbe/intense_sampling.py",
           "gbe/ema_drinking.py"]

doc_url = "https://hgzech.github.io/trr265/"

git_url = "https://github.com/hgzech/trr265/tree/master/"

def custom_doc_links(name): return None
